/*****************************************************************************
 *  Rohde&Schwarz Audio Video Platform Audio Analyzer
 *  LabWindows/CVI Instrument Driver                                     
 *  Original Release: 2013-18-09
    By: Nabil Ouddane
 *      Martin Krcmar
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *       See ChangeLog
 *                                                                           
 *****************************************************************************/
                         
#include <string.h>
#include <stdio.h>  
#include <stdlib.h>
#include <math.h>
#include "rsavpaa.h"

ViStatus _VI_FUNC  rsavpaa_RsClose (ViSession vi);

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsavpaa_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsavpaa_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpaa_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rsavpaa_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsavpaa_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpaa_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;
    
    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));
    
    ATTRLIST(attrList,rsavpaa);  
    checkErr( Rs_SpecificDriverNew (instrSession, "rsavpaa", newOptionString, attrList));
    
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsavpaa);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_SESSION, 0, &instrSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsavpaa_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsavpaa_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSAVPAA_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSAVPAA_ATTR_ID_QUERY_RESPONSE,
                                   0, RSAVPAA_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSAVPAA_VALID_ID_RESPONSE_STRING, strlen(RSAVPAA_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    
    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (instrSession) session. */
    
Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;
        
        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        (void) Rs_Dispose (instrSession); 
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }
    
    return error;
    
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpaa_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpaa_SetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 value)
{
	return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpaa_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpaa_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpaa_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpaa_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsavpaa_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt64 variable.
ViStatus _VI_FUNC rsavpaa_GetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 *value)
{
	return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt64), (ViInt64 *)value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsavpaa_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufSize/Attribute Value parameter.  
/// HIPAR value/The buffer in which the function returns the current value of the 
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many 
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsavpaa_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsavpaa_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/// HIFN  This function queries the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViSession variable.
ViStatus _VI_FUNC rsavpaa_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt32 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpaa_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt64 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpaa_CheckAttributeViInt64 (ViSession instrSession,
                                                   ViChar channelName[],
                                                   ViAttr attributeId,
                                                   ViInt64 value)
{
	return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpaa_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpaa_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpaa_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpaa_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpaa_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpaa_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize >= RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/// HIFN  Selects the audio analyzer tab.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR tab/Selects the wanted tab for audio analyzer.
ViStatus _VI_FUNC rsavpaa_SelectAudioAnalyzerTab (ViSession instrSession,
                                                 ViInt32 board, ViInt32 tab)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_TAB, tab), 3, "Tab");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the audio analyzer signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR signal/Returns the audio analyzer signal state.
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerSignal (ViSession instrSession,
                                                  ViInt32 board, ViInt32 *signal)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_QUERY_AUDIO_ANALYZER_SIGNAL, signal), 3, "Signal");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the selected input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR input/Sets the audio hardware input.
/// HIPAR zoom/Selects the wave display configuration zoom.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerInput (ViSession instrSession,
                                                     ViInt32 board,
                                                     ViString input,
                                                     ViInt32 zoom)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViString(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INPUT, input), 3, "Input");
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_ZOOM, zoom), 4, "Zoom");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the input measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR channelCount/Returns the channel count for the input measurement.
/// HIPAR sampleRate/Returns the sample rate for the input measurement.
/// HIPAR sampleFormat/Returns the sample format for the input measurement.
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerInput (ViSession instrSession,
                                                 ViInt32 board,
                                                 ViInt32 *channelCount,
                                                 ViReal64 *sampleRate,
                                                 ViInt32 *sampleFormat)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_CHANNEL_COUNT, channelCount), 3, "Channel Count");
	
	viCheckParm(rsavpaa_GetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_SAMPLE_RATE, sampleRate), 4, "Sample Rate");
	
	viCheckParm(rsavpaa_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_SAMPLE_FORMAT, sampleFormat), 5, "Sample Format");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the available inputs list for the audio analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR input/Returns the available inputs list.
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerInputsList (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 bufferSize,
                                                      ViChar input[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_GetAttributeViString(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INPUTS_LIST, bufferSize, input), 4, "Input");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Reads the level progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR progress/Returns the level progress.
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerLevelProgress (ViSession instrSession,
                                                         ViInt32 board,
                                                         ViInt32 *progress)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_PROGRESS, progress), 3, "Progress");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the RMS level unit for digital and analog RMS Level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR signalType/Selects the signal type.
/// HIPAR digitalRMSLevel/Selects the digital RMS level unit.
/// HIPAR analogRMSLevel/Selects the digital RMS level unit.
/// HIPAR frequency/Selects the frequency unit.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerLevelUnit (ViSession instrSession,
                                                         ViInt32 board,
                                                         ViInt32 signalType,
                                                         ViInt32 digitalRMSLevel,
                                                         ViInt32 analogRMSLevel,
                                                         ViInt32 frequency)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpaa_invalidViInt32Range (signalType, RSAVPAA_VAL_ANALOG, RSAVPAA_VAL_DIGITAL) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Signal Type");
    };
	
    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
	
	switch(signalType)
		
	{
		case RSAVPAA_VAL_ANALOG:
				   
	    viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
                    RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_RMS_LEVEL_UNIT, analogRMSLevel), 5, "Analog RMS Level");
		
		break;
		
		case RSAVPAA_VAL_DIGITAL:
			
		viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
                    RSAVPAA_ATTR_AUDIO_ANALYZER_RMS_LEVEL_UNIT, digitalRMSLevel), 4, "Digital RMS Level");
		
		break;
	
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Signal Type");
            break;
			
	}
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_UNIT, frequency), 6, "Frequency");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the RMS level and the corresponding frequency for the requested channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR channelNumber/Selects the signal type.
/// HIPAR RMSLevel/Returns the RMS level value for the requested channel.
/// HIPAR frequency/Returns the Frequency corresponding to the RMS level for the requested channel.
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerRMSLevelMeasurementResults (ViSession instrSession,
                                                                     ViInt32 board,
                                                                     ViInt32 channelNumber,
                                                                     ViReal64 *RMSLevel,
                                                                     ViReal64 *frequency)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpaa_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpaa_invalidViInt32Range (channelNumber, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");
    };
	
    sprintf (command, "READ%ld:AUD:LEV:MEAS:VAL? %ld\n", board, channelNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpaa_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (RMSLevel != NULL) 
	*RMSLevel = rsavpaa_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (frequency != NULL) 
	*frequency = rsavpaa_atof (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the level filter configuration name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR index/This control sets the filter index name.
/// HIPAR filter/Sets the level filter configuration name.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerLevelFilters (ViSession instrSession,
                                                            ViInt32 board,
                                                            ViInt32 index,
                                                            ViInt32 filter)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	ViChar      rep_cap1[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));

	sprintf (rep_cap1, ",i%ld", index);
	
    strcat (rep_cap, rep_cap1);
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_FILTERS, filter), 4, "Filter");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Configures the level averaging parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR average/Enables or disables the average level.
/// HIPAR algorithm/Selects the level algorithm.
/// HIPAR averageTime/Sets the level average time value.
/// HIPAR tolerance/Sets the averaging level tolerance.
/// HIPAR settlingTime/Sets the settling time.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerAveragingLevel (ViSession instrSession,
                                                                ViInt32 board,
                                                                ViBoolean average,
                                                                ViInt32 algorithm,
                                                                ViReal64 averageTime,
                                                                ViReal64 tolerance,
                                                                ViReal64 settlingTime)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_AVERAGE_STATE, average), 3, "Average");
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_AVERAGING_ALGORITHM, algorithm), 4, "Algorithm");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_AVERAGE_TIME, averageTime), 5, "Average Time");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_AVERAGING_TOLERANCE, tolerance), 6, "Tolerance");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_LEVEL_AVERAGING_SETTLING_TIME, settlingTime), 7, "Settling Time");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the total harmonic distortion plus Noise THD+N and frequency units.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR THDN/Selects the total harmonic distortion plus Noise THD+N unit.
/// HIPAR digitalRMSLevel/Selects the digital RMS level unit.
/// HIPAR analogRMSLevel/Selects the analog RMS level unit.
/// HIPAR frequency/Selects the frequency unit.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerTHDNUnit (ViSession instrSession,
                                                        ViInt32 board,
														ViInt32 signalType,
                                                        ViInt32 THDN,
                                                        ViInt32 digitalRMSLevel,
                                                        ViInt32 analogRMSLevel,
                                                        ViInt32 frequency)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpaa_invalidViInt32Range (signalType, RSAVPAA_VAL_ANALOG, RSAVPAA_VAL_DIGITAL) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Signal Type");
    };
	
    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_UNIT, THDN), 4, "THDN");
	
	switch(signalType)
		
	{
		case RSAVPAA_VAL_ANALOG:
			
	    viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
                    RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_RMS_LEVEL_UNIT, analogRMSLevel), 6, "Analog RMS Level");
		
		break;
		
		case RSAVPAA_VAL_DIGITAL:
			
		viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
                    RSAVPAA_ATTR_AUDIO_ANALYZER_RMS_LEVEL_UNIT, digitalRMSLevel), 5, "Digital RMS Level");
		
		break;
	
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Signal Type");
            break;
	}
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_UNIT, frequency), 7, "Frequency");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the total harmonic distortion plus Noise frequency range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR frequencyRangeHigh/Sets the high range frequency for total harmonic distortion plus Noise.
/// HIPAR frequencyRangeLow/Sets the low range frequency for total harmonic distortion plus Noise.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerTHDNFrequencyRange (ViSession instrSession,
                                                                    ViInt32 board,
                                                                    ViInt32 frequencyRangeHigh,
                                                                    ViInt32 frequencyRangeLow)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViInt32 (instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_FREQUENCY_RANGE_HIGH, frequencyRangeHigh), 3, "Frequency Range High");
	
	viCheckParm(rsavpaa_SetAttributeViInt32 (instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_FREQUENCY_RANGE_LOW, frequencyRangeLow), 4, "Frequency Range Low");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the total harmonic distortion plus Noise results, for each channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR RMSLevel/Returns the RMS level value for each channel.
/// HIPAR frequency/Returns the Frequency corresponding to the RMS level for the requested channel.
/// HIPAR THDN/eturns the Frequency corresponding to the RMS level for the requested channel.
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerTHDNMeasurementResults (ViSession instrSession,
                                                                 ViInt32 board,
                                                                 ViInt32 channelNumber,
                                                                 ViReal64 *RMSLevel,
                                                                 ViReal64 *frequency,
                                                                 ViReal64 *THDN)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    

	if (rsavpaa_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpaa_invalidViInt32Range (channelNumber, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");
    };
	
    sprintf (command, "READ%ld:AUD:THDN:MEAS:VAL? %ld\n", board, channelNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpaa_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (RMSLevel != NULL) 
	*RMSLevel = rsavpaa_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (frequency != NULL) 
	*frequency = rsavpaa_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (THDN != NULL) 
	*THDN = rsavpaa_atof (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the total harmonic distortion plus Noise progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR progress/Returns the total harmonic distortion plus Noise progress command.
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerTHDNProgress (ViSession instrSession,
                                                        ViInt32 board,
                                                        ViInt32 *progress)
{
 	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_GetAttributeViInt32 (instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_PROGRESS, progress), 3, "Progress");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the total harmonic distortion plus Noise THD+N filter configuration name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR index/This control sets the filter index.
/// HIPAR filter/Sets the level filter configuration name.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerTHDNFilters (ViSession instrSession,
                                                           ViInt32 board,
                                                           ViInt32 index,
                                                           ViInt32 filter)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	ViChar      rep_cap1[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));

	sprintf (rep_cap1, "i%ld", index);
	
	strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap1);
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_FILTERS, filter), 4, "Filter");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Configures the total harmonic distortion plus Noise THD+N averaging parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR average/Enables or disables the average THD+N.
/// HIPAR algorithm/Selects the THD+N algorithm.
/// HIPAR averageTime/Sets the THD+N average time value.
/// HIPAR tolerance/Sets the averaging THD+N tolerance.
/// HIPAR settlingTime/Sets the settling time.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerAveragingTHDN (ViSession instrSession,
                                                               ViInt32 board,
                                                               ViBoolean average,
                                                               ViInt32 algorithm,
                                                               ViReal64 averageTime,
                                                               ViReal64 tolerance,
                                                               ViReal64 settlingTime)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_AVERAGE_STATE, average), 3, "Average");
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_AVERAGING_ALGORITHM, algorithm), 4, "Algorithm");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_AVERAGE_TIME, averageTime), 5, "Average Time");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_AVERAGING_TOLERANCE, tolerance), 6, "Tolerance");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_THDN_AVERAGING_SETTLING_TIME, settlingTime), 7, "Settling Time");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the frequency response display units.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR display/Selects the display unit.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerFrequencyResponseDisplayUnit (ViSession instrSession,
                                                                            ViInt32 board,
                                                                            ViInt32 display)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_RESPONSE_DISPLAY_UNIT, display), 3, "Display");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Enables or disables the channels display for the frequency Response results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR arraySize/Defines number of elements in array Display State.
/// HIPAR displayState/Enables or disables the channels diplay.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerFrequencyResponseDisplayState (ViSession instrSession,
                                                                               ViInt32 board,
                                                                               ViInt32 arraySize,
                                                                               ViBoolean displayState[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    ViInt32     i = 1;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpaa_invalidViInt32Range (board, 1, 8), 2, "Board");
	viCheckParm (rsavpaa_invalidViInt32Range (arraySize, 1, 8), 3, "Array Size");
    
    p2buffer += sprintf (buffer, "CONF%ld:AUD:FRES:SEL ", board);
    
    for (i=0; i<arraySize;i++)
        p2buffer += sprintf (p2buffer, "%s,", displayState[i] ? "ON" : "OFF");

    *p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr( rsavpaa_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the X trace and Y trace results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR xTrace/Returns the X traces values for each channel.
/// HIPAR yTrace/Returns the Y traces values for each channel.
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerFrequencyResponseDisplayTraces (ViSession instrSession,
                                                                         ViInt32 board,
                                                                         ViInt32 channelNumber,
                                                                         ViReal64 xTrace[],
                                                                         ViReal64 yTrace[])
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViInt32  i = 0; 
		
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpaa_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpaa_invalidViInt32Range (channelNumber, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");
    };
	
    sprintf (command, "READ%ld:ARR:AUD:FRES:TRAC? %ld\n", board, channelNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpaa_CheckStatus (instrSession));
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        return VI_ERROR_INV_RESPONSE;
   
    while (p2buf != NULL)
    {
        if (i<=channelNumber)
        {
           xTrace[i++] = rsavpaa_atof (p2buf);
        }
		p2buf = strtok (NULL, ",\n");
	}
	
	while (p2buf != NULL)
    {
        if (i<=channelNumber)
        {
           yTrace[i++] = rsavpaa_atof (p2buf);
        }
		p2buf = strtok (NULL, ",\n");
	}
    checkErr(rsavpaa_CheckStatus (instrSession));
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;	
	
	
}

/// HIFN  Configures the FFT Size and the average count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR FFTSize/Selects the FFT size.
/// HIPAR averageCount/Sets the average count number.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerFrequencyResponseSettings (ViSession instrSession,
                                                                         ViInt32 board,
                                                                         ViInt32 FFTSize,
                                                                         ViInt32 averageCount)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_RESPONSE_FFT_SIZE, FFTSize), 3, "FFT Size");
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_RESPONSE_AVERAGE_COUNT, averageCount), 4, "Average Count");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Reads the Frequency Response progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR progress/Returns the frequency response progress.
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerFrequencyResponseProgress (ViSession instrSession,
                                                                     ViInt32 board,
                                                                     ViInt32 *progress)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_RESPONSE_PROGRESS, progress), 3, "Progress");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the frequency response filters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR index/This control sets the filter index.
/// HIPAR filter/Sets the frequency response filter configuration name.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerFrequencyResponseFilters (ViSession instrSession,
                                                                        ViInt32 board,
                                                                        ViInt32 index,
                                                                        ViInt32 filter)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	ViChar      rep_cap1[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));

	sprintf (rep_cap1, "i%ld", index);
	
	strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap1);
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_RESPONSE_FILTERS, filter), 4, "Filter");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the signal to noise ratio unit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR SNR/Selects the signal to noise unit.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerSignalToNoiseUnit (ViSession instrSession,
                                                                 ViInt32 board,
                                                                 ViInt32 SNR)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_SIGNAL_TO_NOISE_UNIT, SNR), 3, "Signal to Noise Ratio");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Selects the length of the signal and the noise.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR signalLength/Selects the signal length.
/// HIPAR noiseLength/Selects the noise length.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerSignalToNoiseLength (ViSession instrSession,
                                                                   ViInt32 board,
                                                                   ViInt32 signalLength,
                                                                   ViInt32 noiseLength)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_SIGNAL_LENGTH, signalLength), 3, "Signal Length");
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_NOISE_LENGTH, noiseLength), 4, "Noise Length");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the measured SNR for each channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR channelNumber/Selects the signal length.
/// HIPAR SNR/Returns the measured signal to noise ratio for the requested channel.
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerSignalToNoiseMeasurementResults (ViSession instrSession,
                                                                          ViInt32 board,
                                                                          ViInt32 channelNumber,
                                                                          ViReal64 *SNR)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpaa_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpaa_invalidViInt32Range (channelNumber, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");
    };
	
    sprintf (command, "READ%ld:AUD:SNR:MEAS:VAL? %ld\n", board, channelNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpaa_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (SNR != NULL) 
	*SNR = rsavpaa_atof (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the signal to noise command progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR progress/Returns the signal to noise command progress.
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerSignalToNoiseProgress (ViSession instrSession,
                                                                 ViInt32 board,
                                                                 ViInt32 *progress)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_SIGNAL_TO_NOISE_PROGRESS, progress), 3, "Progress");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the signal to noise filters configuration names.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR index/This control sets the filter index .
/// HIPAR filter/Sets the filter configuration name.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerSignalToNoiseFilters (ViSession instrSession,
                                                                    ViInt32 board,
                                                                    ViInt32 index,
                                                                    ViInt32 filter)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	ViChar      rep_cap1[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));

	sprintf (rep_cap1, "i%ld", index);
	
	strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap1);
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_SIGNAL_TO_NOISE_FILTERS, filter), 4, "Filter");
		
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN  Sets the total harmonic distortion plus Noise THD+N and frequency units.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR signalType/Selects the signal type.
/// HIPAR crosstalk/Selects the crosstalk unit.
/// HIPAR digitalRMSLevel/Selects the digital RMS unit.
/// HIPAR analogRMSLevel/Selects the analog RMS unit.
/// HIPAR frequency/Selects the frequency unit.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerCrosstalkUnit (ViSession instrSession,
                                                             ViInt32 board,
                                                             ViInt32 signalType,
                                                             ViInt32 crosstalk,
                                                             ViInt32 digitalRMSLevel,
                                                             ViInt32 analogRMSLevel,
                                                             ViInt32 frequency)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpaa_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpaa_invalidViInt32Range (signalType, RSAVPAA_VAL_ANALOG, RSAVPAA_VAL_DIGITAL) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Signal Type");
    };
	
    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_UNIT, crosstalk), 4, "Crosstalk");
	
	
	switch(signalType)
		
	{
		case RSAVPAA_VAL_ANALOG:
			
	    viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
                    RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_RMS_LEVEL_UNIT, analogRMSLevel), 6, "Analog RMS Level");
		
		break;
		
		case RSAVPAA_VAL_DIGITAL:
			
		viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
                    RSAVPAA_ATTR_AUDIO_ANALYZER_RMS_LEVEL_UNIT, digitalRMSLevel), 5, "Digital RMS Level");
		
		break;
	
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Signal Type");
            break;
	}
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_UNIT, frequency), 7, "Frequency");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the Audio Analyzer Crosstalk Measurement values, the RMS, and the frequency for each or the requested channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR crosstalk/Returns the crosstalk value for the requested channel.
/// HIPAR RMSLevel/Returns the RMS level value for the requested channel.
/// HIPAR frequency/Returns the Frequency value for the requested channel.
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerCrosstalkMeasurementResults (ViSession instrSession,
                                                                      ViInt32 board,
                                                                      ViInt32 channelNumber,
                                                                      ViReal64 *crosstalk,
                                                                      ViReal64 *RMSLevel,
                                                                      ViReal64 *frequency)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpaa_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpaa_invalidViInt32Range (channelNumber, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");
    };
	
    sprintf (command, "READ%ld:AUD:CROS:MEAS:VAL? %ld\n", board, channelNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpaa_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (crosstalk != NULL) 
	*crosstalk = rsavpaa_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (RMSLevel != NULL) 
	*RMSLevel = rsavpaa_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (frequency != NULL) 
	*frequency = rsavpaa_atof (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Configures the crosstalk reference channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR referenceChannel/Selects the reference channel.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerCrosstalkReferenceChannel (ViSession instrSession,
                                                                         ViInt32 board,
                                                                         ViInt32 referenceChannel)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_REFERENCE_CHANNEL, referenceChannel), 3, "Reference Channel");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the crosstalk command progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR progress/Returns the crosstalk command progress.
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerCrosstalkProgress (ViSession instrSession,
                                                             ViInt32 board,
                                                             ViInt32 *progress)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_PROGRESS, progress), 3, "Progress");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the crosstalk filters configuration names.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR index/This control sets the filter index .
/// HIPAR filter/Sets the filter configuration name.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerCrosstalkFilters (ViSession instrSession,
                                                                ViInt32 board,
                                                                ViInt32 index,
                                                                ViInt32 filter)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	ViChar      rep_cap1[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));

	sprintf (rep_cap1, "i%ld", index);
	
	strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap1);
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_FILTERS, filter), 4, "Filter");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Configures the crosstalk averaging parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR average/Enables or disables the average level.
/// HIPAR algorithm/Selects the crosstalk averaging algorithm.
/// HIPAR averageTime/Sets the crosstalk average time value.
/// HIPAR tolerance/Sets the crosstalk averaging level tolerance.
/// HIPAR settlingTime/Sets the settling time.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerAveragingCrosstalk (ViSession instrSession,
                                                                    ViInt32 board,
                                                                    ViBoolean average,
                                                                    ViInt32 algorithm,
                                                                    ViReal64 averageTime,
                                                                    ViReal64 tolerance,
                                                                    ViReal64 settlingTime)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_AVERAGE_STATE, average), 3, "Average");
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_AVERAGING_ALGORITHM, algorithm), 4, "Algorithm");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_AVERAGE_TIME, averageTime), 5, "Average Time");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_AVERAGING_TOLERANCE, tolerance), 6, "Tolerance");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_CROSSTALK_AVERAGING_SETTLING_TIME, settlingTime), 7, "Settling Time");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the interchannel phase, RMS level, and frequency units.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR phase/Selects the phase unit.
/// HIPAR digitalRMSLevel/Selects the digital RMS unit.
/// HIPAR analogRMSLevel/Selects the analog RMS unit.
/// HIPAR frequency/Selects the frequency unit.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerInterchannelPhaseUnit (ViSession instrSession,
                                                                     ViInt32 board,
																	 ViInt32 signalType,
                                                                     ViInt32 phase,
                                                                     ViInt32 digitalRMSLevel,
                                                                     ViInt32 analogRMSLevel,
                                                                     ViInt32 frequency)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpaa_invalidViInt32Range (signalType, RSAVPAA_VAL_ANALOG, RSAVPAA_VAL_DIGITAL) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Signal Type");
    };
	
    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_UNIT, phase), 4, "Phase");
	
	switch(signalType)
		
	{
		case RSAVPAA_VAL_ANALOG:
			
	    viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
                    RSAVPAA_ATTR_AUDIO_ANALYZER_ANALOG_RMS_LEVEL_UNIT, analogRMSLevel), 6, "Analog RMS Level");
		
		break;
		
		case RSAVPAA_VAL_DIGITAL:
			
		viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
                    RSAVPAA_ATTR_AUDIO_ANALYZER_RMS_LEVEL_UNIT, digitalRMSLevel), 5, "Digital RMS Level");
		
		break;
	
		default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Signal Type");
            break;
	}
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_FREQUENCY_UNIT, frequency), 7, "Frequency");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the audio analyzer interchannel phase, RMS level, and frequency measurement values for each channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR phase/Returns the phase value for the requested channel.
/// HIPAR RMSLevel/Returns the RMS level value for the requested channel.
/// HIPAR frequency/Returns the Frequency value for the requested channel.
ViStatus _VI_FUNC rsavpaa_ReadAudioAnalyzerInterchannelPhaseMeasurementResults (ViSession instrSession,
                                                                              ViInt32 board,
                                                                              ViInt32 channelNumber,
                                                                              ViReal64 *phase,
                                                                              ViReal64 *RMSLevel,
                                                                              ViReal64 *frequency)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpaa_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpaa_invalidViInt32Range (channelNumber, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Number");
    };
	
    sprintf (command, "READ%ld:AUD:IPH:MEAS:VAL? %ld\n", board, channelNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpaa_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (phase != NULL) 
	*phase = rsavpaa_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (RMSLevel != NULL) 
	*RMSLevel = rsavpaa_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (frequency != NULL) 
	*frequency = rsavpaa_atof (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Configures the interchannel phase reference channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR referenceChannel/Sets the channel number.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerInterchannelPhaseReferenceChannel (ViSession instrSession,
                                                                                 ViInt32 board,
                                                                                 ViInt32 referenceChannel)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_REFERENCE_CHANNEL, referenceChannel), 3, "Reference Channel");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the interchannel phase command progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR progress/Returns the interchannel phase command progress.
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerInterchannelPhaseProgress (ViSession instrSession,
                                                                     ViInt32 board,
                                                                     ViInt32 *progress)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_PROGRESS, progress), 3, "Progress");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}
/// HIFN  Sets interchannel phase filters configuration names.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR index/This control sets the filter index .
/// HIPAR filter/Sets the filter configuration name.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerInterchannelPhaseFilters (ViSession instrSession,
                                                                        ViInt32 board,
                                                                        ViInt32 index,
                                                                        ViInt32 filter)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[20] = "";
	ViChar      rep_cap1[20] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));

	sprintf (rep_cap1, "i%ld", index);
	
	strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap1);
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_FILTERS, filter), 4, "Filter");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Configures interchannel phase averaging parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR average/Enables otr disables the average interchannel phase.
/// HIPAR algorithm/Selects interchannel phase algorithm.
/// HIPAR averageTime/Sets interchannel phase average time value.
/// HIPAR tolerance/Sets the averaging interchannel phase tolerance.
/// HIPAR settlingTime/Sets the settling time.
ViStatus _VI_FUNC rsavpaa_ConfigureAudioAnalyzerAveragingInterchannelPhase (ViSession instrSession,
                                                                            ViInt32 board,
                                                                            ViBoolean average,
                                                                            ViInt32 algorithm,
                                                                            ViReal64 averageTime,
                                                                            ViReal64 tolerance,
                                                                            ViReal64 settlingTime)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_AVERAGE_STATE, average), 3, "Average");
	
	viCheckParm(rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_AVERAGING_ALGORITHM, algorithm), 4, "Algorithm");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_AVERAGE_TIME, averageTime), 5, "Average Time");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_AVERAGING_TOLERANCE, tolerance), 6, "Tolerance");
	
	viCheckParm(rsavpaa_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_INTERCHANNEL_PHASE_AVERAGING_SETTLING_TIME, settlingTime), 7, "Settling Time");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Starts the Audio Analyzer measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR measurement/Selects measurement to be initiated
ViStatus _VI_FUNC rsavpaa_AudioAnalyzerMeasurementInit (ViSession instrSession,
                                                        ViInt32 board,
                                                        ViInt32 measurement)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));

   viCheckParm (rsavpaa_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_MEASUREMENT_INIT, measurement), 3, "Measurement");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Stops the Audio Analyzer measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
ViStatus _VI_FUNC rsavpaa_AudioAnalyzerMeasurementAbort (ViSession instrSession,
                                                       ViInt32 board)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));

   viCheckParm (rsavpaa_SetAttributeViString(instrSession, rep_cap,
        RSAVPAA_ATTR_AUDIO_ANALYZER_MEASUREMENT_ABORT, VI_NULL), 3, "Stop");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Reads measurement running state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR measurementState/Returns the measurement running state.
ViStatus _VI_FUNC rsavpaa_QueryAudioAnalyzerMeasurementStatus (ViSession instrSession,
                                                             ViInt32 board,
                                                             ViInt32 *measurementState)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpaa_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpaa_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAA_ATTR_QUERY_AUDIO_ANALYZER_MEASUREMENT_STATUS, measurementState), 3, "Measurement State");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
ViStatus _VI_FUNC rsavpaa_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (viWrite (instrSession, (ViBuf) "*RST\n", 5, NULL));
    checkErr( rsavpaa_DefaultInstrSetup (instrSession));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavp_init or 
/// HIPAR instrSession/rsavp_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpaa_ResetSystem (ViSession instrSession, ViInt32 reference)
{
    ViStatus    error   = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	switch (reference)
	{
		case RSAVPAA_VAL_RST_BASE:
			viCheckErr (rsavpaa_SetAttributeViString(instrSession, "",
    	    	RSAVPAA_ATTR_SYSTEM_RESET_BASE, NULL));
			break;
		case RSAVPAA_VAL_RST_ALL:
			viCheckErr (rsavpaa_SetAttributeViString(instrSession, "",
    	    	RSAVPAA_ATTR_SYSTEM_RESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}
    
    checkErr( rsavpaa_DefaultInstrSetup (instrSession));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN The preset sets the parameters to values suitable for local/manual interaction, a reset to values suitable for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpaa_Preset (ViSession instrSession, ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPAA_VAL_RST_BASE:
			viCheckErr (rsavpaa_SetAttributeViString(instrSession, "",
    	    	RSAVPAA_ATTR_SYSTEM_PRESET_BASE, NULL));
			break;
		case RSAVPAA_VAL_RST_ALL:
			viCheckErr (rsavpaa_SetAttributeViString(instrSession, "",
    	    	RSAVPAA_ATTR_SYSTEM_PRESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}
	

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsavpaa_self_test (ViSession instrSession, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     result = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    viCheckErr (rsavpaa_GetAttributeViInt32 (instrSession, NULL, RSAVPAA_ATTR_SELF_TEST_QUERY, &result));
    if (*testResult == 0)
        strcpy (testMessage, "Self-Test Passed.");
    else
        strcpy (testMessage, "Self-Test Failed.");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsavpaa_revision_query (ViSession instrSession, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSAVPAA_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSAVPAA_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsavpaa_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsavpaa_error_query (ViSession instrSession, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSAVPAA_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (rsavpaa_GetAttributeViString( instrSession, "", RSAVPAA_ATTR_SYSTEM_ERROR, RSAVPAA_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%256[^\"]", errCode, errMessage) != 2)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsavpaa_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSAVPAA_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is 
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR length/This control specifies the size of the buffer.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the 
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rsavpaa_SystemVersion(
	ViSession	instrSession,
	ViInt32		length,
	ViChar	  	systemVersion[]
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpaa_GetAttributeViString (instrSession, "",
                                                RSAVPAA_ATTR_SYST_VERSION,
												length,
												systemVersion
                                                ), 2, "System Version");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches ON/OFF state checking of the instrument 
/// HIFN  (reading of the Standard Event Register and checking it for error). All 
/// HIFN  driver functions are using state checking. Switch this function to ON 
/// HIFN  when debug your application. For better bus throughput and instrument 
/// HIFN  performance switch it OFF. As default is state checking switched to ON!
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control selects ON or OFF instrument state checking.
ViStatus _VI_FUNC rsavpaa_setCheckStatus(
	ViSession	instrSession,
	ViBoolean	state
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpaa_SetAttributeViBoolean (instrSession, "",
                                                RS_ATTR_QUERY_INSTRUMENT_STATUS,
												state
                                                ), 2, "State");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Switches the display on or off while the analyzer is in the remote state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR displayUpdate/Selects the local oscillator mode.
ViStatus _VI_FUNC rsavpaa_ConfigureDisplayUpdate (ViSession instrSession,
                                                ViBoolean displayUpdate)
{
    ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm(rsavpaa_SetAttributeViBoolean (instrSession, "",
        RSAVPAA_ATTR_DISPLAY_UPDATE, displayUpdate), 2, "Display Update");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsavpaa_GetError (ViSession instrSession, 
                                     ViStatus *errorCode, 
                                     ViInt32 bufferSize,
                                     ViChar description[])  
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == VI_NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }

    else
        {
            if (sessionProperties == NULL)
            {
                (void) Rs_UnlockSession (instrSession, &locked);
                return 1024;
            }
            else
            {
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
            }
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpaa_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpaa_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        if (description == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
ViStatus _VI_FUNC rsavpaa_ClearError (ViSession instrSession)                                                        
{                                                                                                           
    return Rs_ClearErrorInfo (instrSession);                                                                             
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsavpaa_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_WriteInstrData (instrSession, writeBuffer));
    
    checkErr (rsavpaa_CheckStatus (instrSession));
 
Error:   
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsavpaa_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsavpaa_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rsavpaa_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsavpaa_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsavpaa_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsavpaa_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsavpaa_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsavpaa_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsavpaa_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
ViStatus _VI_FUNC rsavpaa_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rsavpaa_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/// HIFN This function performs all of the drivers clean-up operations
/// HIFN except for closing the RS session.  This function is called by
/// HIFN rsavpaa_close or by an RS class driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpaa_init or 
/// HIPAR instrSession/rsavpaa_InitWithOptions function.
ViStatus _VI_FUNC rsavpaa_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, (ViInt32) sizeof (io), &io));

Error:

    (void) Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, VI_NULL);
    
    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
