/*****************************************************************************
 *  Rohde&Schwarz Audio Video Platform AV Distorsion Analyzer
 *  LabWindows/CVI Instrument Driver                                     
 *  Original Release: 2013-09-18                                 
 *  By: Nabil Ouddane
        Martin Krcmar
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *       See ChangeLog
 *                                                                           
 *****************************************************************************/
                         
#include <string.h>
#include <stdio.h>  
#include <stdlib.h>
#include <math.h>
#include "rsavpavda.h"

ViStatus _VI_FUNC  rsavpavda_RsClose (ViSession vi);

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString SwitchArr[] = {"OFF","ON",VI_NULL};
static ViString PixelParameterArr[] = {"Y","CB","CR","",VI_NULL};
static ViString AVDALogDataArr[] = {"ALL","LVI", VI_NULL};
static ViString AVDADecimalPointArr[] = {"COMM","DOT",VI_NULL};
/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsavpavda_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsavpavda_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpavda_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rsavpavda_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsavpavda_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpavda_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;
    
    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));
    
    ATTRLIST(attrList,rsavpavda);  
    checkErr( Rs_SpecificDriverNew (instrSession, "rsavpavda", newOptionString, attrList));
    
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsavpavda);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_SESSION, 0, &instrSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsavpavda_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsavpavda_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSAVPAVDA_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSAVPAVDA_ATTR_ID_QUERY_RESPONSE,
                                   0, RSAVPAVDA_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSAVPAVDA_VALID_ID_RESPONSE_STRING, strlen(RSAVPAVDA_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    
    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (instrSession) session. */
    
Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;
        
        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        (void) Rs_Dispose (instrSession); 
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }
    
    return error;
    
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpavda_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpavda_SetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 value)
{
	return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpavda_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpavda_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpavda_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpavda_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsavpavda_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt64 variable.
ViStatus _VI_FUNC rsavpavda_GetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 *value)
{
	return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt64), (ViInt64 *)value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsavpavda_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufSize/Attribute Value parameter.  
/// HIPAR value/The buffer in which the function returns the current value of the 
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many 
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsavpavda_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsavpavda_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/// HIFN  This function queries the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViSession variable.
ViStatus _VI_FUNC rsavpavda_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt32 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpavda_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt64 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpavda_CheckAttributeViInt64 (ViSession instrSession,
                                                   ViChar channelName[],
                                                   ViAttr attributeId,
                                                   ViInt64 value)
{
	return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpavda_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpavda_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpavda_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpavda_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpavda_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpavda_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize >= RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/// HIFN  This function selects the operation to be performed on AV Distortion Analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR operation/Selects the operation to be performed on AV Distortion Analyzer.
ViStatus _VI_FUNC rsavpavda_AVDAOperation (ViSession instrSession, ViInt32 board,
                                       ViInt32 operation)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpavda_invalidViInt32Range (operation, RSAVPAVDA_VAL_AVDA_INIT, RSAVPAVDA_VAL_AVDA_CLEAR), 3, "Operation");
	
    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (operation)
    {
        case RSAVPAVDA_VAL_AVDA_INIT:
	
		viCheckErr(rsavpavda_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPAVDA_ATTR_START_AVDA_MEASUREMENT, NULL));
												 
		
		break;
		
		case RSAVPAVDA_VAL_AVDA_ABORT:
	
		viCheckErr(rsavpavda_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPAVDA_ATTR_ABORT_AVDA_MEASUREMENT, NULL));
												 
		
		
		break;
		
		case RSAVPAVDA_VAL_AVDA_CLEAR:
	
		viCheckErr(rsavpavda_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPAVDA_ATTR_CLEAR_AVDA_MEASUREMENT_RESULTS, NULL));
												 
		
		
		break;
	
	 default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Operation");
            break;
			
    }			
	
	checkErr (rsavpavda_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Returns the reference setting state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR state/Returns the reference setting state.
ViStatus _VI_FUNC rsavpavda_QueryAVDAReferenceState (ViSession instrSession,
                                                 ViInt32 board, ViInt32 *state)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpavda_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_REFERENCE_STATE, state), 3, "State");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Configures the synchronization mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR synchronization/Sets the synchronization mode.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDASynchronization (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 synchronization)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_SYNCHRONIZATION, synchronization), 3, "Synchronization");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Configures the Loop APL parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR loopDetectionTime/Sets the time interval for monitoring a signal to detect a loop.
/// HIPAR sectionLeft/Sets the window left size used for loop detection.
/// HIPAR sectionRight/Sets the window right size used for loop detection.
/// HIPAR sectionTop/Sets the window top size used for loop detection.
/// HIPAR sectionBottom/Sets the window bottom size used for loop detection.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDALoopAPL (ViSession instrSession,
                                              ViInt32 board,
                                              ViReal64 loopDetectionTime,
                                              ViReal64 sectionLeft,
                                              ViReal64 sectionRight,
                                              ViReal64 sectionTop,
                                              ViReal64 sectionBottom)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpavda_invalidViReal64Range (sectionLeft, 0.0, 100.0) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Section Left");
    };
	
	if (rsavpavda_invalidViReal64Range (sectionRight, 0.0, 100.0) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Section Right");
	};
	
	if (rsavpavda_invalidViReal64Range (sectionTop, 0.0, 100.0) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "Section Top");
    };
	
	if (rsavpavda_invalidViReal64Range (sectionBottom, 0.0, 100.0) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "Section Bottom");
    };
	
    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpavda_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_LOOP_DETECTION_TIME, loopDetectionTime), 3, "Loop Detection Time");
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:REF:DET:WIND:SIZE %.12lG,%.12lG,%.12lG,%.12lG", board, sectionLeft, sectionRight, sectionTop, sectionBottom));
    
	checkErr(rsavpavda_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Configures the Loop APL parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR allowedMissingFrames/Sets the allowed number of missing frames during recording.
/// HIPAR TCWindowLeft/Sets the window left size used for loop detection.
/// HIPAR TCWindowRight/Sets the window right size used for loop detection.
/// HIPAR TCWindowTop/Sets the window top size used for loop detection.
/// HIPAR TCWindowBottom/Sets the window bottom size used for loop detection.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDALoopTimeCode (ViSession instrSession,
                                                   ViInt32 board,
                                                   ViInt32 allowedMissingFrames,
                                                   ViReal64 TCWindowLeft,
                                                   ViReal64 TCWindowRight,
                                                   ViReal64 TCWindowTop,
                                                   ViReal64 TCWindowBottom)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpavda_invalidViReal64Range (TCWindowLeft, 0.0, 100.0) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "TC Window Left");
    };
	
	if (rsavpavda_invalidViReal64Range (TCWindowRight, 0.0, 100.0) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "TC Window Right");
    };
	
	if (rsavpavda_invalidViReal64Range (TCWindowTop, 0.0, 100.0) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 6, "TC Window Top");
    };
	
	if (rsavpavda_invalidViReal64Range (TCWindowBottom, 0.0, 100.0) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 7, "TC Window Bottom");
    };
	
    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_ALLOWED_MISSING_FRAMES, allowedMissingFrames), 3, "Allowed Missing Frames");
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:REF:TIM:WIND:SIZE %.12lG,%.12lG,%.12lG,%.12lG", board, TCWindowLeft, TCWindowRight, TCWindowTop, TCWindowBottom));
    
	checkErr(rsavpavda_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Returns the playback position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR progress/Returns the playback position.
ViStatus _VI_FUNC rsavpavda_QueryAVDAReferencePlaybackProgress (ViSession instrSession,
                                                            ViInt32 board,
                                                            ViReal64 *progress)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpavda_GetAttributeViReal64(instrSession, rep_cap,
        RSAVPAVDA_ATTR_QUERY_AVDA_REFERENCE_PLAYBACK_PROGRESS, progress), 3, "Progress");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Selects the operation for reference sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR operation/Selects the operation for reference sequence.
ViStatus _VI_FUNC rsavpavda_AVDAReferenceOperation (ViSession instrSession,
                                                ViInt32 board, ViInt32 operation)
{
 
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_REFERENCE_OPERATION, operation), 3, "Operation");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Saves or loads reference sequence to/from a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR operation/Saves or loads reference sequence to/from a file.
/// HIPAR fileName/Selects the file name for loading or saving reference sequence.
ViStatus _VI_FUNC rsavpavda_AVDAReferenceSequenceLoadSave (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViInt32 operation,
                                                       ViString fileName)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpavda_invalidViInt32Range (operation, RSAVPAVDA_VAL_AVDA_REF_LOAD, RSAVPAVDA_VAL_AVDA_REF_SAVE) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Operation");
    };
	
    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (operation)
    {
        case RSAVPAVDA_VAL_AVDA_REF_LOAD:
	
		viCheckParm(rsavpavda_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPAVDA_ATTR_AVDA_REFERENCE_SEQUENCE_LOAD, fileName), 4, "File Name");
												 
		
		break;
		
		case RSAVPAVDA_VAL_AVDA_REF_SAVE:
	
		viCheckParm(rsavpavda_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPAVDA_ATTR_AVDA_REFERENCE_SEQUENCE_STORE,fileName), 4, "File Name");
												 
		
		
		break;
	
	 default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Operation");
            break;
			
    }			
	
	checkErr (rsavpavda_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function enables the AV Distortion Analyzer output an trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR signal/Selects the signal parameter for the measurement.
/// HIPAR loopDetection/Selects the loop detection parameter for the measurement.
/// HIPAR loopDetectionTime/Selects the loop detection time parameter for the measurement.
/// HIPAR loopDetectionTimeLimit/Selects the loop detection time parameter for the measurement.
/// HIPAR frameNumber/Selects the frame number parameter for the measurement.
/// HIPAR cycleNumber/Selects the cycle number parameter for the measurement.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDA (ViSession instrSession, ViInt32 board,
                                       ViBoolean signal, ViBoolean loopDetection,
                                       ViBoolean loopDetectionTime,
                                       ViReal64 loopDetectionTimeLimit,
                                       ViBoolean frameNumber,
                                       ViBoolean cycleNumber)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpavda_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_LIST_SIGNAL_STATE, signal), 3, "Signal");
	
	viCheckParm(rsavpavda_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_LOOP_DETECTION_STATE, loopDetection), 4, "Loop Detection");
	
	viCheckParm(rsavpavda_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_LOOP_DETECTION_TIME_STATE, loopDetectionTime), 5, "Loop Detection Time");
	
	viCheckParm(rsavpavda_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_LOOP_DETECTION_TIME_LIMIT, loopDetectionTimeLimit), 6, "Loop Detection Time Limit");
	
	viCheckParm(rsavpavda_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_FRAME_COUNT_STATE, frameNumber), 7, "Frame Number");

	viCheckParm(rsavpavda_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_CYCLE_NUMBER_STATE, cycleNumber), 8, "Cycle Number");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN Enables the failure point visible error output an trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Selects the visible error parameter for the measurement.
/// HIPAR statusTrace/Selects the visible error to be displayed in the status trace.
/// HIPAR thresholdSSIM/Sets the SSIM threshold.
/// HIPAR thresholdPSNRY/Sets the PSNR Y threshold.
/// HIPAR duration/Sets the visible error duration.
/// HIPAR defaultVideo/Selects the default video type.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAFailurePointVisibleError (ViSession instrSession,
                                                               ViInt32 board,
                                                               ViBoolean enabled,
                                                               ViBoolean statusTrace,
                                                               ViReal64 thresholdSSIM,
                                                               ViReal64 thresholdPSNRY,
                                                               ViInt32 duration)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:FPO:VERR %s, %s", board, SwitchArr[enabled], SwitchArr[statusTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));
	
    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpavda_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_THRESHOLD_SSIM, thresholdSSIM), 5, "Threshold SSIM");
	
	viCheckParm(rsavpavda_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_THRESHOLD_PSNR_Y, thresholdPSNRY), 6, "Threshold PSNRY");
	
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_DURATION, duration), 7, "Duration");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function enables the picture failure point output and trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Selects the picture failure point parameter for the measurement.
/// HIPAR statusTrace/Selects the picture failure point to be displayed in the statu trace.
/// HIPAR length_ofPeriod/Sets the length of period.
/// HIPAR totalNumber_ofPeriods/Sets the total number of periods.
/// HIPAR gapBetweenPeriods/Sets the gap between periods.
/// HIPAR allowedPeriods/Sets the allowed periods with visible error.
/// HIPAR defaultPictures/Selects the default.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAPictureFailurePoint (ViSession instrSession,
                                                          ViInt32 board,
                                                          ViBoolean enabled,
                                                          ViBoolean statusTrace,
                                                          ViInt32 length_ofPeriod,
                                                          ViInt32 totalNumber_ofPeriods,
                                                          ViInt32 gapBetweenPeriods,
                                                          ViInt32 allowedPeriod)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:FPO:PICT %s, %s", board, SwitchArr[enabled], SwitchArr[statusTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));
	
    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_PICTURE_LENGTH_OF_PERIOD, length_ofPeriod), 5, "Length of Period");
	
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_PICTURE_TOTAL_NUMBER_OF_PERIODS, totalNumber_ofPeriods), 6, "Total Number of Periods");
	
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_PICTURE_GAP_BETWEEN_PERIODS, gapBetweenPeriods), 7, "Gap Between Periods");
	
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_PICTURE_ALLOWED_PERIODS, allowedPeriod), 8, "Allowed Periods");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function enables the audio failure point output and trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Selects the audio failure point parameter for the measurement.
/// HIPAR statusTrace/Selects the audio failure point to be displayed in the statu trace.
/// HIPAR duration/Sets the audio failure point duration.
/// HIPAR defaultAudio/Selects the default audio.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAAudioFailurePoint (ViSession instrSession,
                                                        ViInt32 board,
                                                        ViBoolean enabled,
                                                        ViBoolean statusTrace,
                                                        ViInt32 duration)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:FPO:AUD %s,%s", board, SwitchArr[enabled], SwitchArr[statusTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));
	
    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VISIBLE_ERROR_AUDIO_DURATION, duration), 5, "Duration");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function enables the audio failure point output and trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables video freeze output.
/// HIPAR statusTrace/Enables video freeze status trace.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoFreeze (ViSession instrSession,
                                                  ViInt32 board,
                                                  ViBoolean enabled,
                                                  ViBoolean statusTrace)
{
    ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:VFR %s, %s", board, SwitchArr[enabled], SwitchArr[statusTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Enables video dropped frames output, trace and limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables video dropped frames output.
/// HIPAR resultTrace/Enables video dropped frames result trace.
/// HIPAR limit/Sets video dropped frames upper limit.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoDroppedFrames (ViSession instrSession,
                                                         ViInt32 board,
                                                         ViBoolean enabled,
                                                         ViBoolean resultTrace,
                                                         ViInt32 limit)
{
    ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[5] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:FDR %s, %s", board, SwitchArr[enabled], SwitchArr[resultTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));
	
	checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap)); 
	
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VIDEO_DROPPED_FRAMES_LIMIT, limit), 5, "Limit");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Enables video black frame output and trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables video black frame output.
/// HIPAR statusTrace/Enables video black frame status trace.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoblackFrame (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViBoolean enabled,
                                                      ViBoolean statusTrace)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:BFR %s, %s", board, SwitchArr[enabled], SwitchArr[statusTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Enables average video picture level output an trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables average video picture level output.
/// HIPAR resultTrace/Enables average video picture level result trace.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoAveragePictureLevel (ViSession instrSession,
                                                               ViInt32 board,
                                                               ViBoolean enabled,
                                                               ViBoolean resultTrace)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:AVPL %s, %s", board, SwitchArr[enabled], SwitchArr[resultTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Enables video pixel error Y/Cb/Cr output, trace and limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/Selects the parameter.
/// HIPAR enabled/Enables video pixel error output.
/// HIPAR resultTrace/Enables video pixel error result trace.
/// HIPAR limit/Sets video pixel error upper limit.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoPixelError (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 parameter,
                                                      ViBoolean enabled,
                                                      ViBoolean resultTrace,
                                                      ViInt32 limit)
{
    ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[50] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:PXER:%s %s,%s", board, PixelParameterArr[parameter], SwitchArr[enabled], SwitchArr[resultTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));

	checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (parameter)
	{
		case RSAVPAVDA_VAL_AVDA_VIDEO_Y:
			viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        		RSAVPAVDA_ATTR_AVDA_VIDEO_PIXEL_ERROR_LIMIT_Y, limit), 6, "Limit");
			break;
		case RSAVPAVDA_VAL_AVDA_VIDEO_CB:
			viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        		RSAVPAVDA_ATTR_AVDA_VIDEO_PIXEL_ERROR_LIMIT_CB, limit), 6, "Limit");
			break;
		case RSAVPAVDA_VAL_AVDA_VIDEO_CR:
			viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        		RSAVPAVDA_ATTR_AVDA_VIDEO_PIXEL_ERROR_LIMIT_CR, limit), 6, "Limit");
			break;
		default:
	        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Parameter");
			break;
	}
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  enables video peak signal-to-noise ratio Y/Cb/Cr output, trace and limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/Selects the parameter.
/// HIPAR enabled/Enables video peak signal-to-noise ratio output.
/// HIPAR resultTrace/Enable video peak signal-to-noise ratio result trace.
/// HIPAR limit/Sets video peak signal-to-noise ratio lower limit.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoPSNR (ViSession instrSession,
                                                ViInt32 board, 
												ViInt32 parameter,
												ViBoolean enabled,
                                                ViBoolean resultTrace,
                                                ViReal64 limit)
                                                
{
    ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[50] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:PSNR:%s %s, %s", board, PixelParameterArr[parameter],SwitchArr[enabled], SwitchArr[resultTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));

	checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (parameter)
	{
		case RSAVPAVDA_VAL_AVDA_VIDEO_Y:
			viCheckParm(rsavpavda_SetAttributeViReal64(instrSession, rep_cap,
        		RSAVPAVDA_ATTR_AVDA_VIDEO_PSNR_LIMIT_Y, limit), 6, "Limit");
			break;
		case RSAVPAVDA_VAL_AVDA_VIDEO_CB:
			viCheckParm(rsavpavda_SetAttributeViReal64(instrSession, rep_cap,
        		RSAVPAVDA_ATTR_AVDA_VIDEO_PSNR_LIMIT_CB, limit), 6, "Limit");
			break;
		case RSAVPAVDA_VAL_AVDA_VIDEO_CR:
			viCheckParm(rsavpavda_SetAttributeViReal64(instrSession, rep_cap,
        		RSAVPAVDA_ATTR_AVDA_VIDEO_PSNR_LIMIT_CR, limit), 6, "Limit");
			break;
		default:
	        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Parameter");
			break;
	}
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  enables video peak signal-to-noise ratio Y/Cb/Cr output, trace and limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables video structural similarity output.
/// HIPAR resultTrace/Enables video structural similarity result trace.
/// HIPAR limit/Sets video structural similarity lower limit.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoSSIM (ViSession instrSession,
                                                ViInt32 board, ViBoolean enabled,
                                                ViBoolean resultTrace,
                                                ViReal64 limit)
{
    ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:SSIM %s, %s", board, SwitchArr[enabled], SwitchArr[resultTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));

	checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));

	viCheckParm(rsavpavda_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VIDEO_SSIM_LIMIT, limit), 5, "Limit");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Configures the vidoe MOS-V parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables video MOS-V output.
/// HIPAR resultTrace/Enables video MOS-V trace.
/// HIPAR limit/Sets video MOS-V lower limit.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoMOSV (ViSession instrSession,
                                                ViInt32 board, ViBoolean enabled,
                                                ViBoolean resultTrace,
                                                ViReal64 limit)
{
    ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:MOSV %s, %s", board, SwitchArr[enabled], SwitchArr[resultTrace]));
    
	checkErr(rsavpavda_CheckStatus (instrSession));

	checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));

	viCheckParm(rsavpavda_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VIDEO_MOSV_LIMIT, limit), 5, "Limit");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Enables video horizontal shift output and limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables video horizontal shift output.
/// HIPAR lowerLimit/Sets video horizontal shift lower limit.
/// HIPAR upperLimit/Sets video horizontal shift upper limit.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoHorizontalShift (ViSession instrSession,
                                                           ViInt32 board,
	                                                       ViBoolean enabled,
														   ViReal64 lowerLimit,
                                                           ViReal64 upperLimit)
                                                           
{
    ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));

	viCheckParm(rsavpavda_SetAttributeViBoolean (instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VIDEO_HORIZONTAL_SHIFT_STATE, enabled), 3, "Enabled");
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:HSH:LIM %.12lG, %.12lG", board, lowerLimit, upperLimit));
    
	checkErr(rsavpavda_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Enables video vertical shift output and limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables video vertical shift output.
/// HIPAR lowerLimit/Sets video vertical shift lower limit.
/// HIPAR upperLimit/Sets video vertical shift upper limit.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAVideoVerticalShift (ViSession instrSession,
                                                         ViInt32 board,
                                                         ViBoolean enabled,
                                                         ViReal64 lowerLimit,
                                                         ViReal64 upperLimit)
{
    ViStatus	error = VI_SUCCESS;
	ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));

	viCheckParm(rsavpavda_SetAttributeViBoolean (instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_VIDEO_VERTICAL_SHIFT_STATE, enabled), 3, "Enabled");
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LIST:VSH:LIM %.12lG, %.12lG", board, lowerLimit, upperLimit));
    
	checkErr(rsavpavda_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function enables audio loss output and limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables audio loss output for CH 1 to CH 8.
/// HIPAR statusTrace/Enables audio loss status trace for CH 1 to CH 8.
/// HIPAR limit/Sets audio loss upper limit for CH 1 to CH 8.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAAudioLoss (ViSession instrSession,
                                                ViInt32 board,
                                                ViBoolean enabled[],
                                                ViBoolean statusTrace[],
                                                ViInt32 limit[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    ViInt32     i = 0;
    ViString SwitchArr[] = {"OFF","ON",VI_NULL};
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
    
    p2buffer += sprintf (buffer, "CONF%ld:AVDA:LIST:AUD:LOSS ", board);
    
    for (i = 0; i < 8; i++)
            p2buffer += sprintf (p2buffer, "%s,", SwitchArr[enabled[i]]);
	
	for (i = 0; i < 8; i++)
            p2buffer += sprintf (p2buffer, "%s,", SwitchArr[statusTrace[i]]);
	
    
    *p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr( rsavpavda_CheckStatus (instrSession));
	
	p2buffer = buffer + sprintf (buffer, "CONF%ld:AVDA:LIST:AUD:LOSS:LIM ", board);
    
    for (i = 0; i < 8; i++)
            p2buffer += sprintf (p2buffer, "%ld,", limit[i]);
    
	*p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr( rsavpavda_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/// HIFN  This function enables audio level output and limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables audio level output for CH 1 to CH 8.
/// HIPAR resultTrace/Enables audio level result trace for CH 1 to CH 8.
/// HIPAR lowerLimit/Sets audio level lower limit for CH 1 to CH 8.
/// HIPAR upperLimit/Sets audio level upper limit for CH 1 to CH 8.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAAudioLevel (ViSession instrSession,
                                                 ViInt32 board,
                                                 ViBoolean enabled[],
                                                 ViBoolean resultTrace[],
                                                 ViReal64 lowerLimit[],
                                                 ViReal64 upperLimit[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    ViInt32     i = 0;
    ViString SwitchArr[] = {"OFF","ON",VI_NULL};
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
    
    p2buffer += sprintf (buffer, "CONF%ld:AVDA:LIST:AUD:LEV ", board);
    
    for (i = 0; i < 8; i++)
            p2buffer += sprintf (p2buffer, "%s,", SwitchArr[enabled[i]]);
	
	for (i = 0; i < 8; i++)
            p2buffer += sprintf (p2buffer, "%s,", SwitchArr[resultTrace[i]]);
	
    
    *p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr( rsavpavda_CheckStatus (instrSession));
	
	p2buffer = buffer + sprintf (buffer, "CONF%ld:AVDA:LIST:AUD:LEV:LIM ", board);
    
    for (i = 0; i < 8; i++)
		p2buffer += sprintf (p2buffer, "%.12lG,", lowerLimit[i]);

	for (i = 0; i < 8; i++)
        p2buffer += sprintf (p2buffer, "%.12lG,", upperLimit[i]);
    
	*p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr( rsavpavda_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/// HIFN  This function enables audio video delay output and limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR enabled/Enables audio video delay output for CH 1 to CH 8.
/// HIPAR resultTrace/Enables audio video delay result trace for CH 1 to CH 8.
/// HIPAR lowerLimit/Sets audio video delay lower limit for CH 1 to CH 8.
/// HIPAR upperLimit/Sets audio video delay upper limit for CH 1 to CH 8.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAAVDelay (ViSession instrSession,
                                              ViInt32 board, ViBoolean enabled[],
                                              ViBoolean resultTrace[],
                                              ViReal64 lowerLimit[],
                                              ViReal64 upperLimit[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    ViInt32     i = 0;
    ViString SwitchArr[] = {"OFF","ON",VI_NULL};
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    p2buffer += sprintf (buffer, "CONF%ld:AVDA:LIST:AVD ", board);
    
    for (i = 0; i < 8; i++)
        p2buffer += sprintf (p2buffer, "%s,", SwitchArr[enabled[i]]);
	
	for (i = 0; i < 8; i++)
        p2buffer += sprintf (p2buffer, "%s,", SwitchArr[resultTrace[i]]);
	
    *p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr( rsavpavda_CheckStatus (instrSession));
	
	p2buffer = buffer + sprintf (buffer, "CONF%ld:AVDA:LIST:AVD:LIM ", board);
    
    for (i = 0; i < 8; i++)
        p2buffer += sprintf (p2buffer, "%.12lG,", lowerLimit[i]);
	for (i = 0; i < 8; i++)
        p2buffer += sprintf (p2buffer, "%.12lG,", upperLimit[i]);
    
	*p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr( rsavpavda_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;  
}

/// HIFN  This function configures the AVDA input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR externalSync/Determines whether an internal or external synchronization signal is used.
/// HIPAR colorAnalog/Sets the color space for the input signal.
/// HIPAR colorDigital/Sets the color space for the input signal.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDAInput (ViSession instrSession,
                                            ViInt32 board, ViString input,
                                            ViBoolean externalSync,
                                            ViInt32 colorAnalog,
                                            ViInt32 colorDigital)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
   
	viCheckParm(rsavpavda_SetAttributeViString (instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_INPUT_MODULE, input), 3, "Input");
	
	viCheckParm(rsavpavda_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_INPUT_EXTERNAL_SYNCHRONIZATION, externalSync), 4, "External Synchronization");
	
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_INPUT_COLOR_ANALOG, colorAnalog), 5, "Color Analog");
	
	viCheckParm(rsavpavda_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_AVDA_INPUT_COLOR_DIGITAL, colorDigital), 6, "Color Digital");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Queries the catalog of the AV distortion analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR catalog/Returns the catalog of the AV distortion analyzer.
ViStatus _VI_FUNC rsavpavda_QueryAVDAModuleCatalog (ViSession instrSession,
                                               ViInt32 board, ViInt32 bufferSize,
                                               ViChar catalog[])
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpavda_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPAVDA_ATTR_QUERY_AVDA_MODULE_CATALOG, bufferSize,
                                                catalog),4, "Catalog");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function returns whether the input signal could be synchronized to the selected reference signal at least once.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR detectionState/Returns whether the input signal could be synchronized to the selected reference signal at least once.
ViStatus _VI_FUNC rsavpavda_QueryAVDADetectionStatus (ViSession instrSession,
                                                  ViInt32 board,
                                                  ViInt32 *detectionState)
{
 
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpavda_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_QUERY_AVDA_DETECTION_STATUS, detectionState), 3, "Detection State");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function returns the time that was needed to synchronize the input signal to the reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR detectionTime/Returns the time that was needed to synchronize the input signal to the reference.
ViStatus _VI_FUNC rsavpavda_QueryAVDADetectionTime (ViSession instrSession,
                                                ViInt32 board,
                                                ViInt32 *detectionTime)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpavda_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPAVDA_ATTR_QUERY_AVDA_DETECTION_TIME, detectionTime), 3, "Detection Time");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the list parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR signal/Returns signal synchronisation state.
/// HIPAR frameNumberCurrent/Returns current frame number.
/// HIPAR frameNumberWorst/Returns worst frame number.
/// HIPAR cycleNumberCurrent/Returns current cycle number.
/// HIPAR cycleNumberWorst/Returns worst cycle number.
ViStatus _VI_FUNC rsavpavda_QueryAVDAList (ViSession instrSession, ViInt32 board,
                                       ViInt32 *signal,
                                       ViInt32 *frameNumberCurrent,
                                       ViInt32 *frameNumberWorst,
                                       ViInt32 *cycleNumberCurrent,
                                       ViInt32 *cycleNumberWorst)
{
 	ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
	ViChar   *p2buf  = NULL;
	ViString AVDASignalArr[] = {"FAIL","OK", VI_NULL};
	ViChar   buffer [RSAVPAVDA_IO_BUFFER_SIZE] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:SIGN?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
	checkErr (viRead (instrSession, (ViBuf) buffer, RSAVPAVDA_IO_BUFFER_SIZE, NULL));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    if (signal != NULL) *signal = rsavpavda_StringIndex (AVDASignalArr, buffer);

	sprintf (command, "READ%ld:AVDA:LIST:FNUM?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
	checkErr (viRead (instrSession, (ViBuf) buffer, RSAVPAVDA_IO_BUFFER_SIZE, NULL));
    checkErr (rsavpavda_CheckStatus (instrSession));
	
	p2buf = strtok (buffer, ",");
    if (frameNumberCurrent != NULL) 
		*frameNumberCurrent = rsavpavda_atol (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (frameNumberWorst != NULL) 
		*frameNumberWorst = rsavpavda_atol (p2buf);

    
	sprintf (command, "READ%ld:AVDA:LIST:CNUM?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
	checkErr (viRead (instrSession, (ViBuf) buffer, RSAVPAVDA_IO_BUFFER_SIZE, NULL));
    checkErr (rsavpavda_CheckStatus (instrSession));
	
	p2buf = strtok (buffer, ",");
    if (cycleNumberCurrent != NULL) 
		*cycleNumberCurrent = rsavpavda_atol (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (cycleNumberWorst != NULL) 
		*cycleNumberWorst = rsavpavda_atol (p2buf);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns failure point visible error actual, peak and fault frames values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR current/Returns failure point visible error actual value.
/// HIPAR worst/Returns failure point visible error worst value.
/// HIPAR faultFrames/Returns failure point visible error fault frames value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListFailurePointVisibleError (ViSession instrSession,
                                                               ViInt32 board,
                                                               ViInt32 *current,
                                                               ViInt32 *worst,
                                                               ViInt32 *faultFrames)
{
    
 	ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViString AVDASignalArr[] = {"FAIL","OK", VI_NULL};
	

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:FPO:VERR?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    if (current != NULL) *current = rsavpavda_StringIndex (AVDASignalArr, p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    if (worst != NULL) *worst = rsavpavda_StringIndex (AVDASignalArr, p2buf);
	
	p2buf = strtok (NULL, ",");
    	*faultFrames = rsavpavda_atol (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns picture failure point actual, peak and fault frames values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR current/Returns picture failure point actual value.
/// HIPAR worst/Returns picture failure point worst value.
/// HIPAR faultFrames/Returns picture failure point fault frames value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListPictureFailurePoint (ViSession instrSession,
                                                          ViInt32 board,
                                                          ViInt32 *current,
														  ViInt32 *worst,
														  ViInt32 *faultFrames)
                                                          
                                                          
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViString AVDASignalArr[] = {"FAIL","OK", VI_NULL};
	

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:FPO:PICT?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    if (current != NULL) *current = rsavpavda_StringIndex (AVDASignalArr, p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    if (worst != NULL) *worst = rsavpavda_StringIndex (AVDASignalArr, p2buf);
	
	p2buf = strtok (NULL, ",");
    *faultFrames = rsavpavda_atol (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns audio failure point actual, peak and fault frames values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR current/Returns audio failure point actual value.
/// HIPAR worst/Returns audio failure point worst value.
/// HIPAR faultFrames/Returns audio failure point fault frames value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListAudioFailurePoint (ViSession instrSession,
                                                        ViInt32 board,
                                                        ViInt32 *current,
                                                        ViInt32 *worst,
                                                        ViInt32 *faultFrames)
{
    
	ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViString AVDASignalArr[] = {"FAIL","OK", VI_NULL};
	

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:FPO:AUD?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    if (current != NULL) *current = rsavpavda_StringIndex (AVDASignalArr, p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    if (worst != NULL) *worst = rsavpavda_StringIndex (AVDASignalArr, p2buf);
	
	p2buf = strtok (NULL, ",");
    *faultFrames = rsavpavda_atol (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns video freeze actual, peak and fault frames values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR current/Returns video freeze actual value.
/// HIPAR worst/Returns video freeze worst value.
/// HIPAR faultFrames/Returns video freeze fault frames value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoFreeze (ViSession instrSession,
                                                  ViInt32 board,
                                                  ViInt32 *current,
                                                  ViInt32 *worst,
                                                  ViInt32 *faultFrames)
{
 	ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViString AVDASignalArr[] = {"FAIL","OK", VI_NULL};
	

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:VFR?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    if (current != NULL) *current = rsavpavda_StringIndex (AVDASignalArr, p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    if (worst != NULL) *worst = rsavpavda_StringIndex (AVDASignalArr, p2buf);
	
	p2buf = strtok (NULL, ",");
    *faultFrames = rsavpavda_atol (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns video dropped frames actual, peak and default frames values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR current/Returns video dropped frames actual value.
/// HIPAR worst/Returns video dropped frames worst value.
/// HIPAR faultFrames/Returns video dropped frames fault frames value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoDroppedFrames (ViSession instrSession,
                                                         ViInt32 board,
                                                         ViInt32 *current,
                                                         ViInt32 *worst,
                                                         ViInt32 *faultFrames)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:FDR?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    *current = rsavpavda_atol (p2buf);
	
	p2buf = strtok (NULL, ",");
    *worst = rsavpavda_atol (p2buf);
	
	p2buf = strtok (NULL, ",");
    *faultFrames = rsavpavda_atol (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns video black frames actual, peak and fault frames values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR current/Returns video black frames actual value.
/// HIPAR worst/Returns video black frames worst value.
/// HIPAR faultFrames/Returns video black frames fault frames value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoBlackFrames (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViInt32 *current,
                                                       ViInt32 *worst,
                                                       ViInt32 *faultFrames)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViString AVDASignalArr[] = {"FAIL","OK", VI_NULL};
	

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:BFR?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    if (current != NULL) *current = rsavpavda_StringIndex (AVDASignalArr, p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    if (worst != NULL) *worst = rsavpavda_StringIndex (AVDASignalArr, p2buf);
	
	p2buf = strtok (NULL, ",");
    *faultFrames = rsavpavda_atol (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns average video picture level actual value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR current/Returns average video picture level actual value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoAveragePictureLevel (ViSession instrSession,
                                                               ViInt32 board,
                                                               ViReal64 *current)
{
 	ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
	ViChar   buffer[RSAVPAVDA_IO_BUFFER_SIZE] = "";
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:AVPL?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (viRead (instrSession, (ViBuf) buffer, RSAVPAVDA_IO_BUFFER_SIZE, NULL));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    *current = rsavpavda_atof (buffer);
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns video pixel error Y, Cb, Cr actual, peak and default frames values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR parameter/Selects the parameter.
/// HIPAR current/Returns video pixel error actual value.
/// HIPAR worst/Returns video pixel error worst value.
/// HIPAR faultFrames/Returns video pixel error frames value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoPixelError (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 parameter,
                                                      ViInt32 *current,
                                                      ViInt32 *worst,
                                                      ViInt32 *faultFrames)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	if (rsavpavda_invalidViInt32Range (parameter, RSAVPAVDA_VAL_AVDA_VIDEO_Y, RSAVPAVDA_VAL_AVDA_VIDEO_CR) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Parameter");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:PXER:%s?\n", board, PixelParameterArr[parameter]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *current = rsavpavda_atol (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *worst = rsavpavda_atol (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *faultFrames = rsavpavda_atol (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns video signal-to-noise ratio Y, Cb, Cr actual, peak and default frames values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR parameter/Selects the pixel video parameter.
/// HIPAR current/Returns video signal-to-noise ratio actual value.
/// HIPAR worst/Returns video signal-to-noise ratio worst value.
/// HIPAR faultFrames/Returns video signal-to-noise ratio frames value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoPSNR (ViSession instrSession,
                                                ViInt32 board, ViInt32 parameter,
                                                ViReal64 *current,
                                                ViReal64 *worst,
                                                ViInt32 *faultFrames)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (parameter, RSAVPAVDA_VAL_AVDA_VIDEO_Y, RSAVPAVDA_VAL_AVDA_VIDEO_CR) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Parameter");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:PSNR:%s?\n", board, PixelParameterArr[parameter]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *current = rsavpavda_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *worst = rsavpavda_atof (p2buf);
	
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *faultFrames = rsavpavda_atol (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns video structural similarity actual, peak and fault frames values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR current/Returns video structural similarity actual value.
/// HIPAR worst/Returns video structural similarity worst value.
/// HIPAR faultFrames/Returns video structural similarity fault frames value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoSSIM (ViSession instrSession,
                                                ViInt32 board, ViReal64 *current,
                                                ViReal64 *worst,
                                                ViInt32 *faultFrames)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:SSIM?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *current = rsavpavda_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *worst = rsavpavda_atof (p2buf);
	
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *faultFrames = rsavpavda_atol (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns level of video quality actual, peak and fault frames values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR current/Returns level of video quality actual value.
/// HIPAR worst/Returns level of video quality worst value.
/// HIPAR faultFrames/Returns level of video quality fault frames value.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListVideoMOSV (ViSession instrSession,
                                                ViInt32 board, ViReal64 *current,
                                                ViReal64 *worst,
                                                ViInt32 *faultFrames)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    };
	
    sprintf (command, "READ%ld:AVDA:LIST:MOSV?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *current = rsavpavda_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *worst = rsavpavda_atof (p2buf);
	
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        checkErr (RS_ERROR_UNEXPECTED_RESPONSE);
    *faultFrames = rsavpavda_atol (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns audio loss actual and peak values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR arraySize/Sets the array size.
/// HIPAR current/Returns an array of audio loss actual values.
/// HIPAR worst/Returns an array of audio loss worst values.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListAudioLoss (ViSession instrSession,
                                                ViInt32 board, ViInt32 arraySize,
                                                ViInt32 current[],
                                                ViInt32 worst[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViChar     *p2buf  = NULL;
	ViChar     *p2data  = NULL;
	ViUInt32   count = 0;
    ViString AVDASignalArr[] = {"FAIL","OK", VI_NULL};
	ViInt32 	i = 0;
    
    if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }  
	
    sprintf (command, "READ%ld:AVDA:LIST:AUD:LOSS?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpavda_CheckStatus (instrSession));
	
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
   
     while (p2buf != NULL)
    {
        if (i<arraySize)
        {
            if (strcmp (p2buf, "NAV") != 0) 
                current[i++] = rsavpavda_StringIndex (AVDASignalArr, p2buf);
					
        }   
        p2buf = strtok (NULL, ",");
		
		if (i<arraySize)
        {
            if (strcmp (p2buf, "NAV") != 0) 
			    worst[i++] = rsavpavda_StringIndex (AVDASignalArr, p2buf);
            
        }   
        p2buf = strtok (NULL, ",\n");
    
	}
	 
    checkErr(rsavpavda_CheckStatus (instrSession));
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;	
}

/// HIFN  This function returns audio loss actual and peak values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR arraySize/Sets the array size.
/// HIPAR current/Returns an array of audio level actual values.
/// HIPAR worst/Returns an array of audio level worst values.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListAudioLevel (ViSession instrSession,
                                                 ViInt32 board,
                                                 ViInt32 arraySize,
                                                 ViReal64 current[],
                                                 ViReal64 worst[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViChar     *p2buf  = NULL;
	ViChar     *p2data  = NULL;
	ViUInt32   count = 0;
	ViInt32 	i = 0;
    
    if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }  
	
    sprintf (command, "READ%ld:AVDA:LIST:AUD:LEV?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
   
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
   
    while (p2buf != NULL)
    {
        if (i<arraySize)
        {
		   current[i++] = atof (p2buf);	
        }   
        p2buf = strtok (NULL, ",");
		
		if (i<arraySize)
        {
           worst[i++] = atof (p2buf);	                
        }   
        p2buf = strtok (NULL, ",\n");
    
	}
	 
    checkErr(rsavpavda_CheckStatus (instrSession));
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;	
}

/// HIFN  This function returns audio video delay actual and peak values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR arraySize/Sets the array size.
/// HIPAR current/Returns an array of audio video delay actual values.
/// HIPAR worst/Returns an array of audio video delay worst values.
ViStatus _VI_FUNC rsavpavda_QueryAVDAListAVDelay (ViSession instrSession,
                                              ViInt32 board, ViInt32 arraySize,
                                              ViReal64 current[],
                                              ViReal64 worst[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    ViChar     *p2buf  = NULL;
	ViChar     *p2data  = NULL;
	ViUInt32   count = 0;
	ViInt32 	i = 0;
    
    if (rsavpavda_invalidViInt32Range (board, 1, 8) == VI_TRUE)
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }  
	
    sprintf (command, "READ%ld:AVDA:LIST:AVD?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
   
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
   
     while (p2buf != NULL)
    {
        if (i<arraySize)
        {
		   current[i++] = atof (p2buf);	
        }   
        p2buf = strtok (NULL, ",");
		
		if (i<arraySize)
        {
           worst[i++] = atof (p2buf);	                
        }   
        p2buf = strtok (NULL, ",\n");
    
	}
	 
    checkErr(rsavpavda_CheckStatus (instrSession));
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the log options.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR logData/Selects which data is output in the log file.
/// HIPAR decimalPoint/Selects the column delimiter and decimal separator.
ViStatus _VI_FUNC rsavpavda_ConfigureAVDALog (ViSession instrSession,
                                          ViInt32 board, ViInt32 logData,
                                          ViInt32 decimalPoint)
{
    ViStatus error = VI_SUCCESS;
   
    checkErr (Rs_LockSession (instrSession, NULL));

	viCheckParm (rsavpavda_invalidViInt32Range (board, 1, 8), 2, "Board");
	viCheckParm (rsavpavda_invalidViInt32Range (logData, RSAVPAVDA_VAL_AVDA_LOG_ALL_FRAMES, RSAVPAVDA_VAL_AVDA_LOG_LIMIT_VIOLATION), 3, "Log Data");
	viCheckParm (rsavpavda_invalidViInt32Range (decimalPoint, RSAVPAVDA_VAL_AVDA_LOG_COMMA, RSAVPAVDA_VAL_AVDA_LOG_DOT), 4, "Decimal Point");
	
    viCheckErr (viPrintf (instrSession, "CONF%ld:AVDA:LOG:OPT %s, %s\n", board, AVDALogDataArr[logData], AVDADecimalPointArr[decimalPoint]));
								
	checkErr (rsavpavda_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function saves the log into csv file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR fileName/Selects the file name for saving AVDA log.
ViStatus _VI_FUNC rsavpavda_SaveAVDALog (ViSession instrSession, ViInt32 board,
                                     ViString fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";	
	
    checkErr (rsavpavda_hardwareRepCap (instrSession, board, rep_cap));
	viCheckParm (rsavpavda_SetAttributeViString (instrSession, rep_cap,
                                               RSAVPAVDA_ATTR_SAVE_AVDA_LOG,
                                               fileName), 3, "File Name");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
ViStatus _VI_FUNC rsavpavda_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    
	checkErr (viWrite (instrSession, (ViBuf) "*RST\n", 5, NULL));
    
    checkErr( rsavpavda_DefaultInstrSetup (instrSession));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN Resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpavda_ResetSystem (ViSession instrSession,
                                         ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPAVDA_VAL_RST_BASE:
			viCheckErr (rsavpavda_SetAttributeViString(instrSession, "",
        		RSAVPAVDA_ATTR_SYSTEM_RESET_BASE, NULL));
			break;
		case RSAVPAVDA_VAL_RST_ALL:
			viCheckErr (rsavpavda_SetAttributeViString(instrSession, "",
        		RSAVPAVDA_ATTR_SYSTEM_RESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN The preset sets the parameters to values suitable for local/manual interaction, a reset to values suitable for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpavda_Preset (ViSession instrSession, ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPAVDA_VAL_RST_BASE:
			viCheckErr (rsavpavda_SetAttributeViString(instrSession, "",
        		RSAVPAVDA_ATTR_SYSTEM_PRESET_BASE, NULL));
			break;
		case RSAVPAVDA_VAL_RST_ALL:
			viCheckErr (rsavpavda_SetAttributeViString(instrSession, "",
        		RSAVPAVDA_ATTR_SYSTEM_PRESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsavpavda_self_test (ViSession instrSession, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     result = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    viCheckErr (rsavpavda_GetAttributeViInt32 (instrSession, NULL, RSAVPAVDA_ATTR_SELF_TEST_QUERY, &result));
    if (*testResult == 0)
        strcpy (testMessage, "Self-Test Passed.");
    else
        strcpy (testMessage, "Self-Test Failed.");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsavpavda_revision_query (ViSession instrSession, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSAVPAVDA_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSAVPAVDA_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsavpavda_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsavpavda_error_query (ViSession instrSession, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSAVPAVDA_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (rsavpavda_GetAttributeViString( instrSession, "", RSAVPAVDA_ATTR_SYSTEM_ERROR, RSAVPAVDA_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%256[^\"]", errCode, errMessage) != 2)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsavpavda_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSAVPAVDA_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is 
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR length/This control specifies the size of the buffer.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the 
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rsavpavda_SystemVersion(
	ViSession	instrSession,
	ViInt32		length,
	ViChar	  	systemVersion[]
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpavda_GetAttributeViString (instrSession, "",
                                                RSAVPAVDA_ATTR_SYST_VERSION,
												length,
												systemVersion
                                                ), 2, "System Version");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches ON/OFF state checking of the instrument 
/// HIFN  (reading of the Standard Event Register and checking it for error). All 
/// HIFN  driver functions are using state checking. Switch this function to ON 
/// HIFN  when debug your application. For better bus throughput and instrument 
/// HIFN  performance switch it OFF. As default is state checking switched to ON!
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control selects ON or OFF instrument state checking.
ViStatus _VI_FUNC rsavpavda_setCheckStatus(
	ViSession	instrSession,
	ViBoolean	state
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpavda_SetAttributeViBoolean (instrSession, "",
                                                RS_ATTR_QUERY_INSTRUMENT_STATUS,
												state
                                                ), 2, "State");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures the local oscillator.
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvtx_init or 
/// HIPAR instrSession/rsvtx_InitWithOptions function.
/// HIPAR displayUpdate/Selects the local oscillator mode.
ViStatus _VI_FUNC rsavpavda_ConfigureDisplayUpdate (ViSession instrSession,
                                                ViBoolean displayUpdate)
{
    ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm(rsavpavda_SetAttributeViBoolean (instrSession, "",
        RSAVPAVDA_ATTR_DISPLAY_UPDATE, displayUpdate), 2, "Display Update");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsavpavda_GetError (ViSession instrSession, 
                                     ViStatus *errorCode, 
                                     ViInt32 bufferSize,
                                     ViChar description[])  
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == VI_NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }

    else
        {
            if (sessionProperties == NULL)
            {
                (void) Rs_UnlockSession (instrSession, &locked);
                return 1024;
            }
            else
            {
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
            }
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpavda_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpavda_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        if (description == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
ViStatus _VI_FUNC rsavpavda_ClearError (ViSession instrSession)                                                        
{                                                                                                           
    return Rs_ClearErrorInfo (instrSession);                                                                             
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsavpavda_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_WriteInstrData (instrSession, writeBuffer));
    
    checkErr (rsavpavda_CheckStatus (instrSession));
 
Error:   
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsavpavda_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsavpavda_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rsavpavda_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsavpavda_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsavpavda_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsavpavda_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsavpavda_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsavpavda_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsavpavda_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
ViStatus _VI_FUNC rsavpavda_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rsavpavda_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/// HIFN This function performs all of the drivers clean-up operations
/// HIFN except for closing the RS session.  This function is called by
/// HIFN rsavpavda_close or by an RS class driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
ViStatus _VI_FUNC rsavpavda_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, (ViInt32) sizeof (io), &io));

Error:

    (void) Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, VI_NULL);
    
    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
