/*****************************************************************************
 *  Rohde&Schwarz Audio Video Platform HDMI Analyzer
 *  Callbacks
 *
 *  Original Release: October 2013
 *  By: Martin Krcmar
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rsavpha.h"
#include "rsavpha_utility.h"
#include <float.h>
#include <math.h>

#pragma warning (disable:4100) // unreferenced formal parameter

/*****************************************************************************
 * Function: Rs_CheckStatusCallback
 * Purpose:  This function queries the instrument to determine if it has
 *           encountered an error. If the instrument has encountered an
 *           error, this function returns the RS_ERROR_INSTRUMENT_SPECIFIC
 *           error code. This function is called by the
 *           Rs_CheckStatus utility function. The
 *           Rs_SetAttribute and Rs_GetAttribute functions invoke this
 *           function when the optionFlags parameter includes the
 *           RS_VAL_DIRECT_USER_CALL flag.
 *
 *           The user can disable calls to this function by setting the
 *           RSSPECAN_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE.  The driver can
 *           disable the check status callback for a particular attribute by
 *           setting the RS_VAL_DONT_CHECK_STATUS flag.
 *
 *****************************************************************************/
ViStatus rsavpha_CheckStatusCallback (ViSession instrSession)
{
	ViUInt32    ret_cnt;
    ViStatus    error   = VI_SUCCESS;
    ViInt16     esr     = 0;
    ViChar      buffer[256];
    ViInt32     code = 0;
 
    viCheckErr (viWrite (instrSession, (ViBuf) "*STB?\n", 6, NULL));
    viCheckErr (viRead (instrSession, (ViPBuf) buffer, 4, &ret_cnt));
    buffer[ret_cnt] = '\0';
    esr = (ViInt16) atol (buffer);
 
    if ((esr & IEEE_488_2_ERROR_MASK) != 0)
    {
        checkErr (Rs_SetErrorInfo (instrSession, VI_TRUE, RS_ERROR_INSTR_SPECIFIC, code, buffer));
 
        viCheckErr (RS_ERROR_INSTR_SPECIFIC);
    }
 
Error:
 
    return error;
}

/*****************************************************************************
 * Function: Rs_WaitForOPCCallback
 * Purpose:  This function waits until the instrument has finished processing
 *           all pending operations. This function is called by the
 *           Rs_WaitForOPC utility function. The RS engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *
 *****************************************************************************/
ViStatus rsavpha_WaitForOPCCallback (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt16    response    = 0;
    ViInt32     counter     = 0;
    ViUInt16    intfc   = VI_INTF_GPIB;
    ViInt32     opcTimeout = 0;
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0,
        (ViInt32) sizeof (opcTimeout), &opcTimeout));

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    /* intfc specific operations */
    switch (intfc)
        {
        case VI_INTF_ASRL: /* Serial intfc */

            /* Not supported */

        break;
        default: /* Default case */

            /* Check STB for errors. */
            do
            {
                error = viReadSTB (instrSession, &response);

                Rs_Delay (0.001);
                counter++;

                if (counter >= opcTimeout)
                    return RSAVPHA_ERROR_MAX_TIME_EXCEEDED;

                /* Wait while STB is non zero value. Only Error/Event Queue
                   and ESB bits are taken into consideration. */
            
            } while ((response & 0x24) == 0);
            /* (check Error(Event) Queue bit in STB) */
            return ((response & 0x4) != 0) ? RS_ERROR_INSTRUMENT_STATUS : error;
        }

Error:

    return error;
}

/*****************************************************************************
 *- Read Callbacks ----------------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- Write Callbacks ---------------------------------------------------------*
 *****************************************************************************/

 /*****************************************************************************
 *- Check Callbacks ---------------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- Coerce Callbacks --------------------------------------------------------*
 *****************************************************************************/


/*****************************************************************************
 *- Range Table Callbacks ---------------------------------------------------*
 *****************************************************************************/
