/*****************************************************************************
 *  Rohde&Schwarz Video Tester Instrument Driver                               
 *  LabWindows/CVI Instrument Driver                                     
 *  Original Release: 2013-10-11 
 *  By: Martin Krcmar
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *       See ChangeLog
 *                                                                           
 *****************************************************************************/
                         
#include <string.h>
#include <stdio.h>  
#include <stdlib.h>
#include <math.h>
#include "rsavphg.h"

ViStatus _VI_FUNC  rsavphg_RsClose (ViSession vi);

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsavphg_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsavphg_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavphg_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rsavphg_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsavphg_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavphg_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession									   
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;
    
    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));
    
    ATTRLIST(attrList,rsavphg);  
    checkErr( Rs_SpecificDriverNew (instrSession, "rsavphg", newOptionString, attrList));
    
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsavphg);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_SESSION, 0, &instrSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsavphg_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsavphg_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSAVPHG_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSAVPHG_ATTR_ID_QUERY_RESPONSE,
                                   0, RSAVPHG_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSAVPHG_VALID_ID_RESPONSE_STRING, strlen(RSAVPHG_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    
    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (instrSession) session. */
    
Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;
        
        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        (void) Rs_Dispose (instrSession); 
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }
    
    return error;
    
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavphg_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavphg_SetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 value)
{
	return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavphg_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavphg_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavphg_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavphg_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsavphg_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt64 variable.
ViStatus _VI_FUNC rsavphg_GetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 *value)
{
	return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt64), (ViInt64 *)value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsavphg_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufSize/Attribute Value parameter.  
/// HIPAR value/The buffer in which the function returns the current value of the 
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many 
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsavphg_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsavphg_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/// HIFN  This function queries the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViSession variable.
ViStatus _VI_FUNC rsavphg_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt32 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavphg_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt64 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavphg_CheckAttributeViInt64 (ViSession instrSession,
                                                   ViChar channelName[],
                                                   ViAttr attributeId,
                                                   ViInt64 value)
{
	return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavphg_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavphg_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavphg_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavphg_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsavphg_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsavphg_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize >= RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/// HIFN  Returns the HDMI info frame header packet data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR HDMIOut1/Enables or disables the HDMI output 1.
/// HIPAR HDMIOut2/Enables or disables the HDMI output 2.
/// HIPAR HDMIOut3/Enables or disables the HDMI output 3.
/// HIPAR HDMIOut4/Enables or disables the HDMI output 4.
/// HIPAR HDCPOut1/Enables or disables the HDCP (High-bandwidth Digital Content Protection) for the output 1.
/// HIPAR HDCPOut2/Enables or disables the HDCP (High-bandwidth Digital Content Protection) for the output 2.
/// HIPAR HDCPOut3/Enables or disables the HDCP (High-bandwidth Digital Content Protection) for the output 3.
/// HIPAR HDCPOut4/Enables or disables the HDCP (High-bandwidth Digital Content Protection) for the output 4.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorOutputState (ViSession instrSession,
                                                           ViInt32 board,
														   ViBoolean HDMIOut1,
                                                           ViBoolean HDMIOut2,
                                                           ViBoolean HDMIOut3,
                                                           ViBoolean HDMIOut4,
                                                           ViBoolean HDCPOut1,
                                                           ViBoolean HDCPOut2,
                                                           ViBoolean HDCPOut3,
                                                           ViBoolean HDCPOut4)
{
   
	ViStatus error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavphg_invalidViInt32Range (board, 1, 8), 2, "Board");
    
	sprintf (rep_cap, "hw%ld,HDMIPort1", board);
	
    viCheckParm(rsavphg_SetAttributeViBoolean (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_HDMI_OUTPUT,
                                                HDMIOut1), 3, "HDMI Out 1");
	if (HDMIOut1 == VI_TRUE)
	{
		viCheckParm(rsavphg_SetAttributeViBoolean (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_HDCP,
	                                                HDCPOut1), 7, "HDCP Out 1");
	}
	
	sprintf (rep_cap, "hw%ld,HDMIPort2", board);
	
    viCheckParm(rsavphg_SetAttributeViBoolean (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_HDMI_OUTPUT,
                                                HDMIOut2), 4, "HDMI Out 2");
	if (HDMIOut2 == VI_TRUE)
	{
		viCheckParm(rsavphg_SetAttributeViBoolean (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_HDCP,
	                                                HDCPOut2), 8, "HDCP Out 2");
	}
	
	sprintf (rep_cap, "hw%ld,HDMIPort3", board);
	
    viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_HDMI_OUTPUT,
                                                HDMIOut3), 5, "HDMI Out 3");
	if (HDMIOut3 == VI_TRUE)
	{
		viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_HDCP,
	                                                HDCPOut3), 9, "HDCP Out 3");
	}
	
	sprintf (rep_cap, "hw%ld,HDMIPort4", board);
	
    viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_HDMI_OUTPUT,
                                                HDMIOut4), 6, "HDMI Out 4");
	if (HDMIOut4 == VI_TRUE)
	{
		viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_HDCP,
	                                                HDCPOut4), 10, "HDCP Out 4");
	}
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Queries the HPD (hot plug detect) state.
/// HIFN  operations on the storage media of the instrument. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR hotPlugDetect1/Returns the HPD (hot plug detect) state for the output 1.
/// HIPAR hotPlugDetect1/Returns the HPD (hot plug detect) state for the output 2.
/// HIPAR hotPlugDetect1/Returns the HPD (hot plug detect) state for the output 3.
/// HIPAR hotPlugDetect1/Returns the HPD (hot plug detect) state for the output 4.
ViStatus _VI_FUNC rsavphg_QueryHDMIGeneratorOutputHPDState (ViSession instrSession,
	                                                      ViInt32 board, 
                                                          ViInt32 *hotPlugDetect1,
                                                          ViInt32 *hotPlugDetect2,
                                                          ViInt32 *hotPlugDetect3,
                                                          ViInt32 *hotPlugDetect4)
{
    ViStatus error = VI_SUCCESS;
	ViChar      rep_cap[25] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavphg_invalidViInt32Range (board, 1, 8), 2, "Board");
    
	sprintf (rep_cap, "hw%ld,HDMIPort1", board);
	
    viCheckParm(rsavphg_GetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_HOT_PLUG_DETECT_STATE,
                                                hotPlugDetect1), 3, "Hot Plug Detect 1");
	
	sprintf (rep_cap, "hw%ld,HDMIPort2", board);
	
    viCheckParm(rsavphg_GetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_HOT_PLUG_DETECT_STATE,
                                                hotPlugDetect2), 4, "Hot Plug Detect 2");
	
	sprintf (rep_cap, "hw%ld,HDMIPort3", board);
	
    viCheckParm(rsavphg_GetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_HOT_PLUG_DETECT_STATE,
                                                hotPlugDetect3), 5, "Hot Plug Detect 3");
	
	sprintf (rep_cap, "hw%ld,HDMIPort4", board);
	
    viCheckParm(rsavphg_GetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_HOT_PLUG_DETECT_STATE,
                                                hotPlugDetect4), 6, "Hot Plug Detect 4");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the HDMI generator content selection signal pattern.
/// HIFN  operations on the storage media of the instrument. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR pattern/Sets the signal color bar pattern.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalPattern (ViSession instrSession,
                                                             ViInt32 board,
                                                             ViInt32 pattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_PATTERN,
                                                pattern),3, "Pattern");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the HDMI generator content selection signal pattern.
/// HIFN  operations on the storage media of the instrument. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR signalCommand/Selects the signal command.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalCommand (ViSession instrSession,
                                                             ViInt32 board,
                                                             ViInt32 signalCommand)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_COMMAND,
                                                signalCommand),3, "Signal Command");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  Sets the HDMI generator content selection signal pattern.
/// HIFN  operations on the storage media of the instrument. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR type/Selects the HDMI video code index.
/// HIPAR videoCodeIndex/Selects the HDMI video code index (for future use).
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoCode (ViSession instrSession,
                                                               ViInt32 board,
                                                               ViInt32 type,
                                                               ViInt32 videoCodeIndex)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (type)
	{
		case RSAVPHG_VAL_CEA:
			viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CODE,
                                                videoCodeIndex), 4, "Video Code Index");
		break;
		case RSAVPHG_VAL_HDMI:
			viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CODE_HDMI,
                                                videoCodeIndex), 4, "Video Code Index");
		break;
		case RSAVPHG_VAL_VESA:
			viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CODE_VESA,
                                                videoCodeIndex), 4, "Video Code Index");
		break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Type");
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function reads the signal status.
/// HIFN  operations on the storage media of the instrument. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR signalStatus/Reads the signal status.
ViStatus _VI_FUNC rsavphg_ReadHDMIGeneratorSignalStatus (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViInt32 *signalStatus)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavphg_GetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_STATUS,
                                                signalStatus),3, "Signal Status");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN  Sets the video frame rate.
/// HIFN  operations on the storage media of the instrument. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR frameRate/Sets the frame rate.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoFrameRate (ViSession instrSession,
                                                                    ViInt32 board,
                                                                    ViReal64 frameRate)
{
 
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_FRAME_RATE,
                                                frameRate), 3, "Frame Rate");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Selects the HDMI generator video aspect ratio.
/// HIFN  operations on the storage media of the instrument. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR aspectRatio/Selects the ratio aspect ratio.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoAspectRatio (ViSession instrSession,
                                                                      ViInt32 board,
                                                                      ViInt32 aspectRatio)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_ASPECT_RATIO,
                                                aspectRatio),3, "Aspect Ratio");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the video settings.
/// HIFN  operations on the storage media of the instrument. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR pixelEncoding/Selects the video configuration pixel encoding.
/// HIPAR signalingMode/Selects the signaling mode.
/// HIPAR deepColor/Selects the video deep color.
/// HIPAR _3D/Selects the video 3D signaling.
/// HIPAR adjustment/Sets the video configuration adjustment
/// HIPAR resolutionClipping/Selects the video resolution clipping.
/// HIPAR offset/Sets the video configuration color space level offset.
/// HIPAR overallAmplification/Sets for the selected color space (RGB or YCbCr) the overall amplification.
/// HIPAR chromaAmplification/Sets for the YCbCr color space the chroma amplification level.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoSettings (ViSession instrSession,
                                                                   ViInt32 board,
                                                                   ViInt32 pixelEncoding,
                                                                   ViInt32 signalingMode,
                                                                   ViInt32 deepColor,
                                                                   ViInt32 reserved,
                                                                   ViInt32 adjustment,
                                                                   ViInt32 resolutionClipping,
                                                                   ViReal64 offset,
																   ViReal64 overallAmplification,
																   ViReal64 chromaAmplification)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_PIXEL_ENCODING,
                                                pixelEncoding),3, "Pixel Encoding");


    viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_SIGNALING_MODE,
                                                signalingMode),4, "Signaling Mode");
	
	viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_DEEP_COLOR,
                                                deepColor),5, "Deep Color");

	
    /*viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_3D,
                                                _3D),6, "3D");*/
	
    viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_ADJUSTMENT,
                                                adjustment),7, "Adjustment");
	
	
	viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_RESOLUTION_CLIPPING,
                                                resolutionClipping),8, "Resolution Clipping");
	
	if (adjustment == RSAVPHG_VAL_ADJUSTMENT_VARIABLE)
	{
		viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_OFFSET,
	                                                offset),9, "Offset");
	
		if (pixelEncoding == RSAVPHG_VAL_PIXEL_ENCODING_RGB)
		{
			viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_RGB_OVERALL_AMPLIFICATION,
	                                                overallAmplification), 10, "Overall Amplification");
		}
		else
		{
			viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_YCBCR_OVERALL_AMPLIFICATION,
	                                                overallAmplification), 10, "overallAmplification");
	
		    viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CHROMA_AMPLIFICATION,
	                                                chromaAmplification), 11, "Chroma Amplification");
		}
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the video configuration vertical pixel shift.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR pixelShiftVertical/Enables or disables the vertical pixel shift.
/// HIPAR vertical/Sets the video configuration vertical pixel shift value.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoVertical (ViSession instrSession,
                                                                   ViInt32 board,
                                                                   ViBoolean pixelShiftVertical,
                                                                   ViReal64 vertical)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavphg_SetAttributeViBoolean (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_PIXEL_SHIFT_VERTICAL_STATE,
                                                pixelShiftVertical),3, "Pixel Shift Vertical");

	viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_PIXEL_SHIFT_VERTICAL_VALUE,
                                                vertical),4, "Vertical");	
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the video colors levels for RGB color space.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR rLevel/Sets the red level for the RGB color space.
/// HIPAR gLevel/Sets the green level for the RGB color space.
/// HIPAR bLevel/Sets the blue level for the RGB color space.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoRGBLevel (ViSession instrSession,
                                                                   ViInt32 board,
                                                                   ViReal64 rLevel,
                                                                   ViReal64 gLevel,
                                                                   ViReal64 bLevel)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_RED_LEVEL,
                                                rLevel),4, "R Level");	

	viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_GREEN_LEVEL,
                                                gLevel),4, "G Level");
	
	viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_BLUE_LEVEL,
                                                bLevel),4, "B Level");	
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the video colors levels for YCbCr color space. (luma Y,blue Cb, red Cr) 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR yLevel/Sets the luma or Y level for the YCbCr or YPbPr color space.
/// HIPAR cbPbLevel/Sets the Cb/Pb blue minus luma (B-Y).
/// HIPAR crPrLevel/Sets the Cr/Pr red minus luma (R-Y).
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoYCbCrLevel (ViSession instrSession,
                                                                     ViInt32 board,
                                                                     ViReal64 yLevel,
                                                                     ViReal64 cbPbLevel,
                                                                     ViReal64 crPrLevel)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_LUMA_Y_LEVEL,
                                                yLevel),3, "Y Level");	

	viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CB_PB_LEVEL,
                                                cbPbLevel),4, "Cb/Pb Level");
	
	viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_CR_PR_LEVEL,
                                                crPrLevel),5, "Cr/Pr Level");	
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the video configuration parameters to the default settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalVideoDefaultSettings (ViSession instrSession,
                                                                          ViInt32 board)
{
    
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));

   viCheckParm (rsavphg_SetAttributeViString(instrSession, rep_cap,
        RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_VIDEO_DEFAULT_SETTINGS, VI_NULL), 3, "Video Default Settings");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);

    return error;
}

/// HIFN  Configures the HDMI generator audio parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR audio/Selects the audio configuration.
/// HIPAR samplingRate/Selects the audio sampling rate.
/// HIPAR audioLevel/Sets the audio level.
/// HIPAR resolution/Sets the audio resolution.
/// HIPAR channelAllocation_hex/Sets the channel allocation.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalAudioSettings (ViSession instrSession,
                                                                   ViInt32 board,
                                                                   ViInt32 audio,
                                                                   ViInt32 samplingRate,
                                                                   ViReal64 audioLevel,
                                                                   ViInt32 resolution,
                                                                   ViInt32 channelAllocation_hex)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_SELECTION,
                                                audio),3, "Audio");

	if (audio != RSAVPHG_VAL_AUDIO_MUTE)
	{
	    viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_SAMPLING_RATE,
	                                                samplingRate),4, "Sampling Rate");
	
		viCheckParm(rsavphg_SetAttributeViReal64 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_LEVEL,
	                                                audioLevel),5, "Audio Level");

	
	    viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_RESOLUTION,
	                                                resolution),6, "Resolution");
	
	    viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_CHANNEL_ALLOCATION,
	                                                channelAllocation_hex),7, "Channel Allocation Hex");
	}
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Sets the audio channel frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR channelState/Sets audio configuration channel checkboxes.
/// HIPAR channelFrequency/Sets audio configuration channel frequencies.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalAudioChannel (ViSession instrSession,
                                                                  ViInt32 board,
																  ViInt32 port,
                                                                  ViBoolean channelState,
                                                                  ViInt32 channelFrequency)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavphg_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavphg_invalidViInt32Range (port, 1, 8), 3, "Port");
    
	sprintf (rep_cap, "hw%ld,port%ld", board, port);
	
    viCheckParm(rsavphg_SetAttributeViBoolean (instrSession, rep_cap, 
                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_CHANNEL_STATE,
                                                channelState), 4, "Channel State");
	
	if (channelState == VI_TRUE)
	{
		viCheckParm(rsavphg_SetAttributeViInt32 (instrSession, rep_cap, 
	                                                RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_CHANNEL_FREQUENCY,
	                                                channelFrequency), 5, "Channel Frequency");
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Sets the audio configuration to the default settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorSignalAudioDefaultSettings (ViSession instrSession,
                                                                          ViInt32 board)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));

   viCheckParm (rsavphg_SetAttributeViString(instrSession, rep_cap,
        RSAVPHG_ATTR_HDMI_GENERATOR_SIGNAL_AUDIO_DEFAULT_SETTINGS, VI_NULL), 3, "Audio Default Settings");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Selects the HDMI output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
/// HIPAR output/Selects the HDMI output.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorEDIDOutput (ViSession instrSession,
                                                          ViInt32 board,
                                                          ViInt32 output)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));

    viCheckParm (rsavphg_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPHG_ATTR_HDMI_GENERATOR_EDID_OUTPUT, output), 3, "Output");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  Saves the HDMI generator EDID output data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
ViStatus _VI_FUNC rsavphg_SaveHDMIGeneratorEDIDOutput (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViString fileName)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));

    viCheckParm (rsavphg_SetAttributeViString (instrSession, rep_cap,
        RSAVPHG_ATTR_SAVE_HDMI_GENERATOR_EDID_OUTPUT, fileName), 3, "File Name");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
	return error; 
}

/// HIFN  Updates EDID Reader output details.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR board/This control selects the board.
ViStatus _VI_FUNC rsavphg_UpdateHDMGeneratorEDIDOutput (ViSession instrSession,
                                                      ViInt32 board)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));

    viCheckErr (rsavphg_SetAttributeViString(instrSession, rep_cap,
        RSAVPHG_ATTR_UPDATE_HDMI_GENERATOR_EDID_OUTPUT, VI_NULL));

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function selects the output connector of the R&S VTC that is 
/// HIFN  connected with the DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR output/Selects the output connector of the R&S VTC that is connected with the 
/// HIPAR output/DUT.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorCECOutput(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32	output
)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));

    viCheckParm (rsavphg_SetAttributeViInt32 (instrSession, rep_cap,
        RSAVPHG_ATTR_HDMI_GENERATOR_CEC_OUTPUT, output), 3, "Output");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;  
}


/// HIFN  This function sets the logical address of the R&S VTC and of the DUT 
/// HIFN  that should receive the CEC messages sent by the R&S VTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR logicalAddress/Sets the logical address of the R&S VTC.
/// HIPAR destination/Sets the logical address of the DUT that should receive the CEC 
/// HIPAR destination/messages sent by the R&S VTC.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorCECLogicalAddress(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32	logicalAddress,
	ViInt32	destination
)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));

    viCheckParm (rsavphg_SetAttributeViInt32 (instrSession, rep_cap,
        RSAVPHG_ATTR_HDMI_GENERATOR_CEC_LOGICAL_ADDRESS, logicalAddress), 3, "Logical Address");
	
	viCheckParm (rsavphg_SetAttributeViInt32 (instrSession, rep_cap,
        RSAVPHG_ATTR_HDMI_GENERATOR_CEC_DESTINATION, destination), 4, "Destination");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;     
}


/// HIFN  Sets the selected operation code (opcode) of the CEC command that is 
/// HIFN  sent by the R&S VTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR operationCode/Sets the selected operation code (opcode) of the CEC command that is 
/// HIPAR operationCode/sent by the R&S VTC.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorCECOperationCode(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32	operationCode
)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));

    viCheckParm (rsavphg_SetAttributeViInt32 (instrSession, rep_cap,
        RSAVPHG_ATTR_HDMI_GENERATOR_CEC_OPERATION_CODE, operationCode), 3, "Operation Code");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;     
}


/// HIFN  This function sets operation code parameter length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR operationCode/Sets the selected operation code (opcode) of the CEC command that is 
/// HIPAR operationCode/sent by the R&S VTC.
/// HIPAR parameter/This control selects parameter 1.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorCECOperationCodeParameterLength(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32	operationCode,
	ViInt32	length
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
	ViReal64 	discrete;
    ViChar   	*p2Code = NULL;
	int 		i = 0;
	
    checkErr (Rs_LockSession (instrSession, NULL));
	
	do
    {
        discrete = rsavphg_rngHDMIAnalyzerCECOperationCode.rangeValues[i].discreteOrMinValue;
        p2Code = rsavphg_rngHDMIAnalyzerCECOperationCode.rangeValues[i].cmdString;

        if (p2Code == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Operation Code");

        if (fabs(discrete - operationCode) <= 0.01) break;

        i++;

    } while (i != 0);

    viCheckParm (rsavphg_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavphg_invalidViInt32Range (length, 0, 14), 4, "Length");
    
    sprintf (command, "CONF%ld:HDG:CEC:OPC:PAR:LENG %s,%ld", board, p2Code, length);
		
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rsavphg_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;     
}


/// HIFN  This function sets operation code parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR operationCode/Sets the selected operation code (opcode) of the CEC command that is 
/// HIPAR operationCode/sent by the R&S VTC.
/// HIPAR parameter1/This control selects parameter 1.
/// HIPAR parameter2/This control selects parameter 2.
ViStatus _VI_FUNC rsavphg_ConfigureHDMIGeneratorCECOperationCodeParameter (ViSession instrSession,
                                                                           ViInt32 board,
                                                                           ViInt32 operationCode,
                                                                           ViInt32 index,
                                                                           ViInt32 parameter)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
	ViReal64 	discrete;
    ViChar   	*p2Code = NULL;
	int 		i = 0;
	
    checkErr (Rs_LockSession (instrSession, NULL));
	
	do
    {
        discrete = rsavphg_rngHDMIAnalyzerCECOperationCode.rangeValues[i].discreteOrMinValue;
        p2Code = rsavphg_rngHDMIAnalyzerCECOperationCode.rangeValues[i].cmdString;

        if (p2Code == RS_RANGE_TABLE_END_STRING)
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Operation Code");

        if (fabs(discrete - operationCode) <= 0.01) break;

        i++;

    } while (i != 0);

    viCheckParm (rsavphg_invalidViInt32Range (board, 1, 8), 2, "Board");
    viCheckParm (rsavphg_invalidViInt32Range (index, 0, 14), 4, "Index");
    
    sprintf (command, "CONF%ld:HDG:CEC:OPC:PAR %s,%ld,#H%x", board, p2Code, index, parameter);
		
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rsavphg_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}


/// HIFN  This function selects the operation to be performed on the HDMI 
/// HIFN  analyzer CTS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR operation/This control selects the operation.
ViStatus _VI_FUNC rsavphg_HDMIGeneratorCECOperation(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32	operation
)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rsavphg_invalidViInt32Range (operation, RSAVPHG_VAL_HDMI_START, RSAVPHG_VAL_HDMI_STOP), 3, "Operation");
    
    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	switch (operation)
    {
        case RSAVPHG_VAL_HDMI_START:
			viCheckErr(rsavphg_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHG_ATTR_HDMI_GENERATOR_CEC_INIT, NULL));
			break;
		
		case RSAVPHG_VAL_HDMI_STOP:
			viCheckErr(rsavphg_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHG_ATTR_HDMI_GENERATOR_CEC_STOP, NULL));
		break;
	 default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER,3,"Operation");
            break;
			
    }			
	
	checkErr (rsavphg_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;    
}


/// HIFN  This function sends a command with the selected opcode and parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
ViStatus _VI_FUNC rsavphg_HDMIGeneratorCECSend(
	ViSession	instrSession,
	ViInt32	board
)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	checkErr (Rs_LockSession (instrSession, VI_NULL)); 
    
    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckErr(rsavphg_SetAttributeViString (instrSession, rep_cap, 
                                                 RSAVPHG_ATTR_HDMI_GENERATOR_CEC_SEND, NULL));
	
	checkErr (rsavphg_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;     
}


/// HIFN  This function saves the protocol trace into a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR fileName/Selects the file name for saving captured data.
ViStatus _VI_FUNC rsavphg_HDMIGeneratorCECDataSave(
	ViSession	instrSession,
	ViInt32	board,
	ViString	fileName
)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));

    viCheckParm (rsavphg_SetAttributeViString (instrSession, rep_cap,
        RSAVPHG_ATTR_HDMI_GENERATOR_CEC_DATA_STORE, fileName), 3, "File Name");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;        
}


/// HIFN  This function returns the HDMI analyzer CEC test output report count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR count/This control returns the HDMI analyzer CEC test output report count.
ViStatus _VI_FUNC rsavphg_QueryHDMIGeneratorCECTestOutputReportCount(
	ViSession	instrSession,
	ViInt32	board,
	ViInt32*	count
)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavphg_hardwareRepCap (instrSession, board, rep_cap));

    viCheckParm (rsavphg_GetAttributeViInt32 (instrSession, rep_cap,
        RSAVPHG_ATTR_HDMI_GENERATOR_CEC_TEST_OUTPUT_REPORT_COUNT, count), 3, "count");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;     
}


/// HIFN  This function returns the HDMI analyzer CEC test output report data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR board/This control selects the board.
/// HIPAR index/This control sets the index.
/// HIPAR arraySize/This control sets the array size.
/// HIPAR lineNumber/This control returns line number referring to TestOutput scroll view. 
/// HIPAR lineNumber/First line has #0
/// HIPAR referenceNumber/This control returns reference number of text message
/// HIPAR textMessage/This control returns text message.
/// HIPAR optionalParameters/This control returns the optional parameters.
ViStatus _VI_FUNC rsavphg_QueryHDMIGeneratorCECTestOutputReport (ViSession instrSession,
                                                                 ViInt32 board,
                                                                 ViInt32 index,
                                                                 ViInt32 textMessageBufferSize,
                                                                 ViInt32 arraySize,
                                                                 ViInt32 *lineNumber,
                                                                 ViInt32 *referenceNumber,
                                                                 ViChar textMessage[],
                                                                 ViInt32 optionalParameters[],
                                                                 ViInt32 *returnedValues)
{
	ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPHG_IO_BUFFER_SIZE] = "";
	ViChar   buffer[RSAVPHG_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViInt32 i = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	viCheckParm (rsavphg_invalidViInt32Range (board, 1, 8), 2, "Board");
	
    sprintf (command, "READ%ld:HDG:CEC:REP? %ld\n", board, index);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavphg_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (lineNumber != NULL) 
	*lineNumber = atoi (p2buf);

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return error;
    if (referenceNumber != NULL) 
		sscanf (p2buf, "%x", &referenceNumber);
	
    p2buf = strtok (NULL, ",");
	if (strspn (p2buf, "\"") != 0)
	{
		sscanf (p2buf, "\"%[^\"]", buffer);
		strncpy (textMessage, buffer, textMessageBufferSize);
		p2buf = strtok (NULL, ",");
	}
	
	while (p2buf && i<arraySize)
    {
	   optionalParameters[i++] = atol (p2buf);
	   p2buf = strtok (NULL, ",");
    }
	
	*returnedValues = i;
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
ViStatus _VI_FUNC rsavphg_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)
        {
        viCheckErr( viPrintf (instrSession, "*RST\n"));
        }

    checkErr( rsavphg_DefaultInstrSetup (instrSession));
    checkErr( rsavphg_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;

}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavphg_ResetSystem (ViSession instrSession,
                                       ViInt32 reference)
{
	ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPHG_VAL_RST_BASE:
			viCheckErr (rsavphg_SetAttributeViString(instrSession, "",
        		RSAVPHG_ATTR_SYSTEM_RESET_BASE, NULL));
			break;
		case RSAVPHG_VAL_RST_ALL:
			viCheckErr (rsavphg_SetAttributeViString(instrSession, "",
        		RSAVPHG_ATTR_SYSTEM_RESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN The preset sets the parameters to values suitable for local/manual interaction, a reset to values suitable for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavphg_Preset (ViSession instrSession, ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPHG_VAL_RST_BASE:
			viCheckErr (rsavphg_SetAttributeViString(instrSession, "",
        		RSAVPHG_ATTR_SYSTEM_PRESET_BASE, NULL));
			break;
		case RSAVPHG_VAL_RST_ALL:
			viCheckErr (rsavphg_SetAttributeViString(instrSession, "",
        		RSAVPHG_ATTR_SYSTEM_PRESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsavphg_self_test (ViSession instrSession, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     result = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    viCheckErr (rsavphg_GetAttributeViInt32 (instrSession, NULL, RSAVPHG_ATTR_SELF_TEST_QUERY, &result));
    if (*testResult == 0)
        strcpy (testMessage, "Self-Test Passed.");
    else
        strcpy (testMessage, "Self-Test Failed.");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsavphg_revision_query (ViSession instrSession, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSAVPHG_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSAVPHG_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsavphg_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsavphg_error_query (ViSession instrSession, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSAVPHG_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (rsavphg_GetAttributeViString( instrSession, "", RSAVPHG_ATTR_SYSTEM_ERROR, RSAVPHG_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%256[^\"]", errCode, errMessage) != 2)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsavphg_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSAVPHG_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is 
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR length/This control specifies the size of the buffer.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the 
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rsavphg_SystemVersion(
	ViSession	instrSession,
	ViInt32		length,
	ViChar	  	systemVersion[]
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavphg_GetAttributeViString (instrSession, "",
                                                RSAVPHG_ATTR_SYST_VERSION,
												length,
												systemVersion
                                                ), 2, "System Version");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches ON/OFF state checking of the instrument 
/// HIFN  (reading of the Standard Event Register and checking it for error). All 
/// HIFN  driver functions are using state checking. Switch this function to ON 
/// HIFN  when debug your application. For better bus throughput and instrument 
/// HIFN  performance switch it OFF. As default is state checking switched to ON!
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control selects ON or OFF instrument state checking.
ViStatus _VI_FUNC rsavphg_setCheckStatus(
	ViSession	instrSession,
	ViBoolean	state
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavphg_SetAttributeViBoolean (instrSession, "",
                                                RS_ATTR_QUERY_INSTRUMENT_STATUS,
												state
                                                ), 2, "State");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function configures the local oscillator.
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR displayUpdate/Selects the local oscillator mode.
ViStatus _VI_FUNC rsavphg_ConfigureDisplayUpdate (ViSession instrSession,
                                                ViBoolean displayUpdate)
{
    ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm(rsavphg_SetAttributeViBoolean (instrSession, "",
        RSAVPHG_ATTR_DISPLAY_UPDATE, displayUpdate), 2, "Display Update");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsavphg_GetError (ViSession instrSession, 
                                     ViStatus *errorCode, 
                                     ViInt32 bufferSize,
                                     ViChar description[])  
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
 
    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));
 
    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == VI_NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }
 
    else
        {
            if (sessionProperties == NULL)
            {
                (void) Rs_UnlockSession (instrSession, &locked);
                return 1024;
            }
            else
            {
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
            }
        }
 
    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavphg_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }
 
    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavphg_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }
 
    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }
 
    actualSize = (ViInt32) (strlen(errorMessage) + 1);
 
    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        if (description == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }
 
    if (errorCode)
        {
        *errorCode = primary;
        }
 
Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
ViStatus _VI_FUNC rsavphg_ClearError (ViSession instrSession)                                                        
{                                                                                                           
    return Rs_ClearErrorInfo (instrSession);                                                                             
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsavphg_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_WriteInstrData (instrSession, writeBuffer));
    
    checkErr (rsavphg_CheckStatus (instrSession));
 
Error:   
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsavphg_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsavphg_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rsavphg_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsavphg_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsavphg_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsavphg_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsavphg_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsavphg_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsavphg_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
ViStatus _VI_FUNC rsavphg_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rsavphg_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/// HIFN This function performs all of the drivers clean-up operations
/// HIFN except for closing the RS session.  This function is called by
/// HIFN rsavphg_close or by an RS class driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavphg_init or 
/// HIPAR instrSession/rsavphg_InitWithOptions function.
ViStatus _VI_FUNC rsavphg_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, (ViInt32) sizeof (io), &io));

Error:

    (void) Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, VI_NULL);
    
    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
