/*****************************************************************************
 *  Rohde&Schwarz Video Tester Instrument Driver                               
 *  LabWindows/CVI Instrument Driver                                     
 *  Original Release: 2013-09-18                                 
 *  By: Nabil Ouddane
 *      Martin Krcmar
 
 *  Modification History:                                                    
 *                                                                           
 *       See ChangeLog
 *                                                                           
 *****************************************************************************/
                         
#include <string.h>
#include <stdio.h>  
#include <stdlib.h>
#include <math.h>
#include "rsavpva.h"

ViStatus _VI_FUNC  rsavpva_RsClose (ViSession vi);

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString VideoAnalyzerParameterArr[] = {"TIME","PIX",VI_NULL};
static ViString CursorArr[] = {"","C1","C2","C3","C4","C5","C6","C7","C8","C9","C10",VI_NULL};
static ViString ScopeViewArr[] = {"YGR","CBBL","CRR","CCVS","SYNC",VI_NULL};
static ViString VideoColorArr[] = {"BLAC","BLUE","CYAN","GRE","MAG","RED","WHIT","YELL",VI_NULL};
static ViString VideoTraceVector[] = {"XTR","YTR",VI_NULL};
static ViString ReferencePresetArr[] = {"Base","All",VI_NULL};
/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rsavpva_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsavpva_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpva_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rsavpva_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsavpva_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsavpva_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;
    
    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));
    
    ATTRLIST(attrList,rsavpva);  
    checkErr( Rs_SpecificDriverNew (instrSession, "rsavpva", newOptionString, attrList));
    
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rsavpva);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_SESSION, 0, &instrSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rsavpva_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rsavpva_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSAVPVA_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSAVPVA_ATTR_ID_QUERY_RESPONSE,
                                   0, RSAVPVA_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSAVPVA_VALID_ID_RESPONSE_STRING, strlen(RSAVPVA_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    
    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (instrSession) session. */
    
Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;
        
        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        (void) Rs_Dispose (instrSession); 
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }
    
    return error;
    
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpva_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpva_SetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 value)
{
	return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpva_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpva_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpva_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function sets the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsavpva_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rsavpva_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViInt64 variable.
ViStatus _VI_FUNC rsavpva_GetAttributeViInt64 (ViSession instrSession,
                                                 ViChar channelName[],
                                                 ViAttr attributeId,
                                                 ViInt64 *value)
{
	return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt64), (ViInt64 *)value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rsavpva_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the 
/// HIPAR bufSize/Attribute Value parameter.  
/// HIPAR value/The buffer in which the function returns the current value of the 
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many 
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsavpva_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rsavpva_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/// HIFN  This function queries the value of a ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the 
/// HIPAR channelName/attribute is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a 
/// HIPAR value/ViSession variable.
ViStatus _VI_FUNC rsavpva_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt32 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpva_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a ViInt64 
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or 
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpva_CheckAttributeViInt64 (ViSession instrSession,
                                                   ViChar channelName[],
                                                   ViAttr attributeId,
                                                   ViInt64 value)
{
	return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpva_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpva_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpva_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN  This function checks the validity of a value you specify for a 
/// HIFN  ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name 
/// HIPAR channelName/of the channel on which to check the attribute value. If the attribute 
/// HIPAR channelName/is not channel-based, then pass VI_NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value which you want to verify as a valid value for the 
/// HIPAR value/attribute.
ViStatus _VI_FUNC rsavpva_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpva_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsavpva_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize >= RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/// HIFN  Selects the wanted tab for the video analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvtx_init or 
/// HIPAR instrSession/rsvtx_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR tab/Selects the wanted tab for the video analyzer.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzer (ViSession instrSession,
                                                  ViInt32 board, ViInt32 tab)
{
    ViStatus error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";	
	
	checkErr (rsavpva_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm (rsavpva_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSAVPVA_ATTR_VIDEO_ANALYZER_TAB,
                                               tab), 3, "Tab");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function configures the video analyzer input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR input/Selects the input signal for the measurement.
/// HIPAR freeze/Configures whether measurement results are retained for documentation or kept in the display for an extended period of time.
/// HIPAR average/Configures the averaging of the video input signal.
/// HIPAR colorAnalog/Sets the color space for the input signal.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerInput (ViSession instrSession,
                                                     ViInt32 board,
                                                     ViString input,
                                                     ViBoolean freeze,
                                                     ViInt32 average,
                                                     ViInt32 colorAnalog)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpva_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpva_SetAttributeViString(instrSession, rep_cap,
        RSAVPVA_ATTR_VIDEO_ANALYZER_INPUT, input), 3, "Input");
	
	viCheckParm(rsavpva_SetAttributeViBoolean(instrSession, rep_cap,
        RSAVPVA_ATTR_VIDEO_ANALYZER_EXTERNAL_FREEZE, freeze), 4, "Freeze");
	
	viCheckParm(rsavpva_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPVA_ATTR_VIDEO_ANALYZER_AVERAGE_COUNT, average), 5, "Average");
	
	viCheckParm(rsavpva_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPVA_ATTR_VIDEO_ANALYZER_COLOR_ANALOG, colorAnalog), 6, "Color Analog");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  Queries the catalog of the video analyzer input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR catalog/Returns the catalog of the video analyzer.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerCatalog (ViSession instrSession,
                                                   ViInt32 board,
                                                   ViInt32 bufferSize,
                                                   ViChar catalog[])
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpva_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpva_GetAttributeViString (instrSession, rep_cap, 
                                                RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_CATALOG, bufferSize,
                                                catalog),4, "Catalog");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the video analyzer parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR resolution/Returns current video resolution (horizontal/vertical) of the measured input signal.
/// HIPAR standard/Returns the video standard of the measured input signal.
/// HIPAR averagingLevel/Returns the current averaging level if averaging is enabled.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerInput (ViSession instrSession,
                                                 ViInt32 board,
                                                 ViInt32 bufferSize,
                                                 ViChar resolution[],
                                                 ViInt32 *standard,
                                                 ViReal64 *averagingLevel,
												 ViInt32 *color)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpva_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpva_GetAttributeViString (instrSession, rep_cap, 
                             RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_INPUT_RESOLUTION, bufferSize, resolution), 4, "Resolution");
	
	viCheckParm(rsavpva_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_INPUT_STANDARD, standard), 5, "Standard");
	
	viCheckParm(rsavpva_GetAttributeViReal64(instrSession, rep_cap,
        RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_AVERAGING_LEVEL, averagingLevel), 6, "Averaging Level");
	
	viCheckParm(rsavpva_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_DIGITAL_COLOR, color), 7, "Color");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error; 
}

/// HIFN  This function returns the video format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/Selects the board.
/// HIPAR TVMaxLine/Returns the maximum lines.
/// HIPAR TVFrameRate/Returns frame rate.
/// HIPAR lineWidth/Returns the line width.
/// HIPAR pixelClock/Returns pixel clock.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerFormat (ViSession instrSession,
                                                  ViInt32 board,
                                                  ViInt32 *TVMaxLine,
                                                  ViReal64 *TVFrameRate,
                                                  ViReal64 *lineWidth,
                                                  ViReal64 *pixelClock)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPVA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	
    sprintf (command, "READ%ld:VID:FORM?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpva_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    *TVMaxLine = rsavpva_atol (p2buf);
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    *TVFrameRate = rsavpva_atof (p2buf);
	
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    *lineWidth = rsavpva_atof (p2buf);
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    *pixelClock = rsavpva_atof (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function configures the Sin x/x scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR scale/Selects the scale.
/// HIPAR leftBottom/Sets the left (X-axis) or bottom (Y-axis) value.
/// HIPAR rightTop/Sets the right (X-axis) or top (Y-axis) value.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerSinX_XScale (ViSession instrSession,
                                                           ViInt32 board,
                                                           ViInt32 scale,
                                                           ViReal64 leftBottom,
                                                           ViReal64 rightTop)
{
    ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, VI_NULL));    
	
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	
	if (rsavpva_invalidViInt32Range (scale, RSAVPVA_VAL_VIDEO_SCALE_X, RSAVPVA_VAL_VIDEO_SCALE_Y_GROUP_DELAY))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Scale");
    }
	
	switch (scale)
	{
		case RSAVPVA_VAL_VIDEO_SCALE_X:
			
		viCheckErr (viPrintf (instrSession, "DISP%ld:VID:SINXd:TRAC:X %.12lG, %.12lG", board, leftBottom, rightTop));
    
	    checkErr(rsavpva_CheckStatus (instrSession));
		
		break;
		
		case RSAVPVA_VAL_VIDEO_SCALE_Y_AMPLITUDE:
			
		viCheckErr (viPrintf (instrSession, "DISP%ld:VID:SINX:AMPL:TRAC:Y %.12lG, %.12lG", board, leftBottom, rightTop));
    
	    checkErr(rsavpva_CheckStatus (instrSession));
	
		break;
		
		case RSAVPVA_VAL_VIDEO_SCALE_Y_GROUP_DELAY:
			
		viCheckErr (viPrintf (instrSession, "DISP%ld:VID:SINX:GDEL:TRAC:Y %.12lG, %.12lG", board, leftBottom, rightTop));
    
	    checkErr(rsavpva_CheckStatus (instrSession));
	
		break;
		
	}
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function configures the content of graph pane.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR scopeView/Selects the content of graph pane.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerScopeView (ViSession instrSession,
                                                         ViInt32 board,
                                                         ViInt32 scopeView)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpva_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpva_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPVA_ATTR_VIDEO_ANALYZER_SCOPE_VIEW, scopeView), 3, "Scope View");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function configures the X axis parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/Selects the parameter.
/// HIPAR line/Sets the X axis line.
/// HIPAR position/Sets the X axis position.
/// HIPAR span/Sets the X axis span.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerScopeXAxis (ViSession instrSession,
                                                          ViInt32 board,
                                                          ViInt32 parameter,
                                                          ViReal64 line,
                                                          ViReal64 position,
                                                          ViReal64 span)
{
    ViStatus    error = VI_SUCCESS;
   
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
    
	if (rsavpva_invalidViInt32Range (parameter, RSAVPVA_VAL_TIME, RSAVPVA_VAL_PIXEL))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Parameter");
    }
		
	viCheckErr (viPrintf (instrSession, "DISP%ld:SCOP:TRAC:X:LPOS:%s %.12lG,%.12lG,%.12lG", board, VideoAnalyzerParameterArr[parameter], line, position, span));
    
	checkErr(rsavpva_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the Y axis parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR position/Sets the Y axis position.
/// HIPAR span/Sets the Y axis span.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerScopeYAxis (ViSession instrSession,
                                                          ViInt32 board,
                                                          ViReal64 position,
                                                          ViReal64 span)
{
    ViStatus    error = VI_SUCCESS;
   
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
    
	viCheckErr (viPrintf (instrSession, "DISP%ld:SCOP:TRAC:Y:POS %.12lG, %.12lG", board, position, span));
    
	checkErr(rsavpva_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the scope cursor type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR cursor/Selects the cursor.
/// HIPAR cursorType/Sets the cursor function.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerScopeCursor (ViSession instrSession,
                                                           ViInt32 board,
                                                           ViInt32 cursor,
                                                           ViInt32 cursorType)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
	ViChar      rep_cap1[15] = "";

	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	if (rsavpva_invalidViInt32Range (cursor, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Cursor");
    }
	
    checkErr (rsavpva_hardwareRepCap (instrSession, board, rep_cap));
	
	sprintf (rep_cap1, "%s", CursorArr[cursor]);
	
    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap1);
    
	viCheckParm(rsavpva_SetAttributeViInt32(instrSession, rep_cap,
        RSAVPVA_ATTR_VIDEO_ANALYZER_CURSOR_TYPE, cursorType), 3, "Cursor Type");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function configures the cursor X axis parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR cursor/Selects the cursor.
/// HIPAR parameter/Selects the parameter.
/// HIPAR line/Sets the cursor line.
/// HIPAR position/Sets the cursor position.
/// HIPAR span/Sets the span.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerScopeCursorXAxis (ViSession instrSession,
                                                                ViInt32 board,
                                                                ViInt32 cursor,
                                                                ViInt32 parameter,
                                                                ViReal64 line,
                                                                ViReal64 position,
																ViReal64 span)
{
    ViStatus    error = VI_SUCCESS;
   
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	if (rsavpva_invalidViInt32Range (cursor, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Cursor");
    }
    if (rsavpva_invalidViInt32Range (parameter, RSAVPVA_VAL_TIME, RSAVPVA_VAL_PIXEL))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Parameter");
    }
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:SCOP:TRAC:CURS%ld:X:LPOS:%s %.12lG, %.12lG, %.12lG", board, cursor, VideoAnalyzerParameterArr[parameter], line, position,span));
    
	checkErr(rsavpva_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function returns the trace values of the scope.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR scopeView/Selects the trace to read.
/// HIPAR bufferSize/This control sets the buffer size.
/// HIPAR data/Returns the trace values of the scope.
/// HIPAR returnedValues/Returns the number of values returned.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerScopeTrace (ViSession instrSession,
                                                      ViInt32 board,
                                                      ViInt32 scopeView,
                                                      ViInt32 bufferSize,
                                                      ViReal64 data[],
                                                      ViInt32 *returnedValues)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPVA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViInt32 	i = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	
	if (rsavpva_invalidViInt32Range (scopeView, RSAVPVA_VAL_VIDEO_SCOPE_TRACE_GREEN, RSAVPVA_VAL_VIDEO_SCOPE_TRACE_SYNC))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Scope View");
    }
	
    sprintf (command, "READ%ld:ARR:SCOP:TRAC? %s\n", board, ScopeViewArr[scopeView]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpva_CheckStatus (instrSession));
    
   p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        return VI_ERROR_INV_RESPONSE;
   
     while (p2buf != NULL)
    {
        if (i<bufferSize)
        {
           data[i++] = rsavpva_atof (p2buf);
        }
		p2buf = strtok (NULL, ",\n");
	}
	if (returnedValues) *returnedValues=i;
	
    checkErr(rsavpva_CheckStatus (instrSession));
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;	
}

/// HIFN  This function returns the cursor level of the scope.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR cursor/Selects the cursor.
/// HIPAR scopeView/Selects the cursor parameter.
/// HIPAR position/Returns the cursor position
/// HIPAR slope/Returns the cursor slope.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerScopeCursor (ViSession instrSession,
                                                       ViInt32 board,                                                    
													   ViInt32 cursor,
	                                                   ViInt32 scopeView,
                                                       ViReal64 *position,
													   ViReal64 *slope)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPVA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	if (rsavpva_invalidViInt32Range (cursor, 1, 2))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Cursor");
    }
	if (rsavpva_invalidViInt32Range (scopeView, RSAVPVA_VAL_VIDEO_SCOPE_Y_GREEN, RSAVPVA_VAL_VIDEO_SCOPE_SYNC))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Scope View");
    }
	
    sprintf (command, "READ%ld:SCOP:TRAC:CURS%ld? %s\n", board, cursor, ScopeViewArr[scopeView]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpva_CheckStatus (instrSession));
    
    p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    *position = rsavpva_atof (p2buf);
	
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        checkErr (VI_ERROR_INV_RESPONSE);
    *slope = rsavpva_atof (p2buf);
	
    checkErr(rsavpva_CheckStatus (instrSession));
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the cursor level of the scope.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR parameter/Selects the parameter.
/// HIPAR line/Sets the cursor line.
/// HIPAR position/Sets the cursor position.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerVectorCursor (ViSession instrSession,
                                                            ViInt32 board,
                                                            ViInt32 parameter,
                                                            ViInt32 line,
                                                            ViReal64 position)
{
    ViStatus    error = VI_SUCCESS;
   
	checkErr (Rs_LockSession (instrSession, VI_NULL));
	
    if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	if (rsavpva_invalidViInt32Range (parameter, RSAVPVA_VAL_TIME, RSAVPVA_VAL_PIXEL))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Parameter");
    }
	if (rsavpva_invalidViInt32Range (line, 1, 525))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Line");
    }
	
	viCheckErr (viPrintf (instrSession, "CONF%ld:VECT:TRAC:CURS:LPOS:%s %ld, %.12lG", board, VideoAnalyzerParameterArr[parameter], line, position));
    
	checkErr(rsavpva_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function configures the vector gain and phase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR gain/Sets the gain to expand the vector trace in the diagram./
/// HIPAR phase/Sets the phase to rotate the vector trace in the diagram.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerVectorGainPhase (ViSession instrSession,
                                                               ViInt32 board,
                                                               ViReal64 gain,
                                                               ViReal64 phase)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpva_hardwareRepCap (instrSession, board, rep_cap));
    
	viCheckParm(rsavpva_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPVA_ATTR_VIDEO_ANALYZER_VECTOR_GAIN, gain), 3, "Gain");
	
	viCheckParm(rsavpva_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPVA_ATTR_VIDEO_ANALYZER_VECTOR_PHASE, phase), 4, "Phase");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the color bar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR color/Selects the color.
/// HIPAR luminanceAmplitude/Returns luminance amplitude.
/// HIPAR chrominanceAmplitude/Returns chrominance amplitude.
/// HIPAR chrominancePhase/Returns chrominance phase.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerVectorColor (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViInt32 color,
                                                       ViReal64 *luminanceAmplitude,
                                                       ViReal64 *chrominanceAmplitude,
                                                       ViReal64 *chrominancePhase)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPVA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	if (rsavpva_invalidViInt32Range (color, RSAVPVA_VAL_VECTOR_COLOR_BLACK, RSAVPVA_VAL_VECTOR_COLOR_YELLOW))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Color");
    }
	
    sprintf (command, "READ%ld:VECT:CBAR:%s?\n", board, VideoColorArr[color]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpva_CheckStatus (instrSession));
    
	p2buf = strtok (p2data, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (luminanceAmplitude != NULL) 
	*luminanceAmplitude = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (chrominanceAmplitude != NULL) 
	*chrominanceAmplitude = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (chrominancePhase != NULL) 
	*chrominancePhase = rsavpva_atof (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns current cursor measurement values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR luminanceAmplitude/Selects the color.
/// HIPAR chrominanceAmplitude/Returns luminance amplitude.
/// HIPAR chrominancePhase/Returns chrominance amplitude.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerVectorCursor (ViSession instrSession,
                                                        ViInt32 board,
                                                        ViReal64 *luminanceAmplitude,
                                                        ViReal64 *chrominanceAmplitude,
                                                        ViReal64 *chrominancePhase)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPVA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }

    sprintf (command, "READ%ld:VECT:CURS?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpva_CheckStatus (instrSession));
    
	p2buf = strtok (p2data, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (luminanceAmplitude != NULL) 
	*luminanceAmplitude = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (chrominanceAmplitude != NULL) 
	*chrominanceAmplitude = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (chrominancePhase != NULL) 
	*chrominancePhase = rsavpva_atof (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the current cursor measurement values V+ and V- for CCVS input signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR vLuminanceAmplitudeN/Returns V+ luminance amplitude.
/// HIPAR vChrominanceAmplitudeN/Returns V+ chrominance amplitude.
/// HIPAR vChrominancePhaseN/Returns V+ chrominance phase.
/// HIPAR vLuminanceAmplitudeP/Returns V- luminance amplitude.
/// HIPAR vChrominanceAmplitudeP/Returns V- chrominance amplitude.
/// HIPAR vChrominancePhaseP/Returns V- chrominance phase.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerVectorCursorPhase (ViSession instrSession,
                                                             ViInt32 board,
                                                             ViReal64 *vLuminanceAmplitudeN,
                                                             ViReal64 *vChrominanceAmplitudeN,
                                                             ViReal64 *vChrominancePhaseN,
                                                             ViReal64 *vLuminanceAmplitudeP,
                                                             ViReal64 *vChrominanceAmplitudeP,
                                                             ViReal64 *vChrominancePhaseP)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPVA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }

    sprintf (command, "READ%ld:VECT:CURS:VPH?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpva_CheckStatus (instrSession));
    
	p2buf = strtok (p2data, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (vLuminanceAmplitudeN != NULL) 
	*vLuminanceAmplitudeN = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (vChrominanceAmplitudeN != NULL) 
	*vChrominanceAmplitudeN = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (vChrominancePhaseN != NULL) 
	*vChrominancePhaseN = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (vLuminanceAmplitudeP != NULL) 
	*vLuminanceAmplitudeP = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (vChrominanceAmplitudeP != NULL) 
	*vChrominanceAmplitudeP = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (vChrominancePhaseP != NULL) 
	*vChrominancePhaseP = rsavpva_atof (p2buf);
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the vector video format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR TVMaxLine/Returns maximum lines.
/// HIPAR TVFrameRate/Returns frame rate.
/// HIPAR lineWidth/Returns line width.
/// HIPAR pixelClock/Returns pixel clock.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerVectorVideoFormat (ViSession instrSession,
                                                             ViInt32 board,
                                                             ViInt32 *TVMaxLine,
                                                             ViReal64 *TVFrameRate,
                                                             ViReal64 *lineWidth,
                                                             ViReal64 *pixelClock)
{
    ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPVA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;

    checkErr (Rs_LockSession (instrSession, VI_NULL));
    
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	
    sprintf (command, "READ%ld:VECT:VID:FORM?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpva_CheckStatus (instrSession));
    
	p2buf = strtok (p2data, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (TVMaxLine != NULL) 
	*TVMaxLine = atol (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (TVFrameRate != NULL) 
	*TVFrameRate = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (lineWidth != NULL) 
	*lineWidth = rsavpva_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
	if (p2buf == NULL)
		viCheckErr (VI_ERROR_INV_RESPONSE);
    if (pixelClock != NULL) 
	*pixelClock = rsavpva_atof (p2buf);

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN  This function returns the vector trace data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR trace/Selects the trace to read.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR vectorTrace/Returns the trace values of the vector.
/// HIPAR lineTrace/Returns the line trace values.
/// HIPAR returnedValues/Returns the number of values returned.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerVectorTrace (ViSession instrSession,
                                                       ViInt32 board,
                                                       ViInt32 trace,
                                                       ViInt32 bufferSize,
                                                       ViReal64 vectorTrace[],
													   ViReal64 lineTrace[],
                                                       ViInt32 *returnedValues)
{
	ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPVA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViInt32 	i = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	if (rsavpva_invalidViInt32Range (trace, RSAVPVA_VAL_VIDEO_TRACE_X, RSAVPVA_VAL_VIDEO_TRACE_Y))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Trace");
    }
	
    sprintf (command, "READ%ld:ARR:VECT:TRAC:%s?\n", board, VideoTraceVector[trace]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpva_CheckStatus (instrSession));
		
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        return VI_ERROR_INV_RESPONSE;
   
     while (p2buf != NULL)
    {
        if (i<bufferSize)
        {
           vectorTrace[i++] = rsavpva_atof (p2buf);
        }
		p2buf = strtok (NULL, ",\n");
	}
	if (returnedValues) *returnedValues=i;
	
    checkErr(rsavpva_CheckStatus (instrSession));

	sprintf (command, "READ%ld:ARR:VECT:%s?\n", board, VideoTraceVector[trace]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpva_CheckStatus (instrSession));
		
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        return VI_ERROR_INV_RESPONSE;
   
     while (p2buf != NULL)
    {
        if (i<bufferSize)
        {
           lineTrace[i++] = rsavpva_atof (p2buf);
        }
		p2buf = strtok (NULL, ",\n");
	}
	if (returnedValues) *returnedValues=i;
	
    checkErr(rsavpva_CheckStatus (instrSession));
	
Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;	
}

/// HIFN  This function configures the video waveform cursor center and span parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR center/Sets the cursor center position.
/// HIPAR span/Sets the cursor span.
ViStatus _VI_FUNC rsavpva_ConfigureVideoAnalyzerWaveformCursorCenterSpan (ViSession instrSession,
                                                                          ViInt32 board,
                                                                          ViInt32 cursor,
                                                                          ViReal64 center,
                                                                          ViReal64 span)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
	ViChar      rep_cap1[10] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpva_invalidViInt32Range (cursor, 1, 10), 3, "Cursor");
	checkErr (rsavpva_hardwareRepCap (instrSession, board, rep_cap));
	sprintf (rep_cap1, "%s", CursorArr[cursor]);
	
    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap1);
    
	viCheckParm(rsavpva_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPVA_ATTR_VIDEO_ANALYZER_WAVEFORM_CURSOR_CENTER, center), 4, "Center");
	
	viCheckParm(rsavpva_SetAttributeViReal64(instrSession, rep_cap,
        RSAVPVA_ATTR_VIDEO_ANALYZER_WAVEFORM_CURSOR_SPAN, span), 5, "Span");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the video waveform measurement state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR state/Sets the cursor center position.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerWaveformState (ViSession instrSession,
                                                         ViInt32 board,
                                                         ViInt32 *state)
{
    ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    checkErr (rsavpva_hardwareRepCap (instrSession, board, rep_cap));
	
	viCheckParm(rsavpva_GetAttributeViInt32(instrSession, rep_cap,
        RSAVPVA_ATTR_QUERY_VIDEO_ANALYZER_WAVEFORM_STATE, state), 3, "State");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}

/// HIFN  This function returns the video waveform measurement trace data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR board/This control selects the board.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR data/Returns the trace values.
/// HIPAR returnedValues/Returns the number of values returned.
ViStatus _VI_FUNC rsavpva_QueryVideoAnalyzerWaveformTrace (ViSession instrSession,
                                                         ViInt32 board,
                                                         ViInt32 bufferSize,
                                                         ViReal64 data[],
                                                         ViInt32 *returnedValues)
{
	ViStatus error = VI_SUCCESS;
	ViChar   command[RSAVPVA_IO_BUFFER_SIZE] = "";
    ViUInt32 count = 0;
	ViChar   *p2buf  = NULL;
	ViChar   *p2data  = NULL;
	ViInt32 	i = 0;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));    
    
	if (rsavpva_invalidViInt32Range (board, 1, 8))
    {
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Board");
    }
	
    sprintf (command, "READ%ld:ARR:VID:MEAS:WAV:TRAC?\n", board);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength (instrSession, &p2data, &count));
    checkErr (rsavpva_CheckStatus (instrSession));
		
	p2buf = strtok (p2data, ",");
    if (p2buf == NULL)
        return VI_ERROR_INV_RESPONSE;
   
     while (p2buf != NULL)
    {
        if (i<bufferSize)
        {
           data[i++] = rsavpva_atof (p2buf);
        }
		p2buf = strtok (NULL, ",\n");
	}
	if (returnedValues) *returnedValues=i;
	
    checkErr(rsavpva_CheckStatus (instrSession));

Error:
    if (p2data) free (p2data);
    Rs_UnlockSession(instrSession, VI_NULL);    
    return error;
}

/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
ViStatus _VI_FUNC rsavpva_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	checkErr (viWrite (instrSession, (ViBuf) "*RST\n", 5, NULL));

    checkErr( rsavpva_DefaultInstrSetup (instrSession));

Error:
    (void) Rs_UnlockSession (instrSession, VI_NULL);
    return error;
}

/// HIFN The preset sets the parameters to values suitable for local/manual interaction, a reset to values suitable for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpva_Preset (ViSession instrSession, ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPVA_VAL_RST_BASE:
			viCheckErr (rsavpva_SetAttributeViString(instrSession, "",
    	    	RSAVPVA_ATTR_SYSTEM_PRESET_BASE, NULL));
			break;
		case RSAVPVA_VAL_RST_ALL:
			viCheckErr (rsavpva_SetAttributeViString(instrSession, "",
    	    	RSAVPVA_ATTR_SYSTEM_PRESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}
	

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN The preset sets the parameters to values suitable for local/manual interaction, a reset to values suitable for remote operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR reference/Selects the reference of preset.
ViStatus _VI_FUNC rsavpva_ResetSystem (ViSession instrSession,
                                       ViInt32 reference)
{
    ViStatus	error = VI_SUCCESS;
	
    checkErr (Rs_LockSession (instrSession, VI_NULL));

	switch (reference)
	{
		case RSAVPVA_VAL_RST_BASE:
			viCheckErr (rsavpva_SetAttributeViString(instrSession, "",
    	    	RSAVPVA_ATTR_SYSTEM_RESET_BASE, NULL));
			break;
		case RSAVPVA_VAL_RST_ALL:
			viCheckErr (rsavpva_SetAttributeViString(instrSession, "",
    	    	RSAVPVA_ATTR_SYSTEM_RESET_ALL, NULL));
			break;
		default:
			viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Reference");
			break;
	}
	

Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error; 
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsavpva_self_test (ViSession instrSession, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViInt32     result = 0;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    viCheckErr (rsavpva_GetAttributeViInt32 (instrSession, NULL, RSAVPVA_ATTR_SELF_TEST_QUERY, &result));
    if (*testResult == 0)
        strcpy (testMessage, "Self-Test Passed.");
    else
        strcpy (testMessage, "Self-Test Failed.");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsavpva_revision_query (ViSession instrSession, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSAVPVA_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSAVPVA_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rsavpva_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsavpva_error_query (ViSession instrSession, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSAVPVA_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr (rsavpva_GetAttributeViString( instrSession, "", RSAVPVA_ATTR_SYSTEM_ERROR, RSAVPVA_IO_BUFFER_SIZE, buffer));
        if  (sscanf (buffer, "%ld,\"%256[^\"]", errCode, errMessage) != 2)
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);

        }
    else
        {
            /* Simulate Error Query */
        *errCode = 0;
        strcpy (errMessage, "No error.");
        }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rsavpva_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSAVPVA_ERROR_CODES_AND_MSGS,
            {VI_NULL, VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is 
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR length/This control specifies the size of the buffer.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the 
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rsavpva_SystemVersion(
	ViSession	instrSession,
	ViInt32		length,
	ViChar	  	systemVersion[]
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpva_GetAttributeViString (instrSession, "",
                                                RSAVPVA_ATTR_SYST_VERSION,
												length,
												systemVersion
                                                ), 2, "System Version");

Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN  This function switches ON/OFF state checking of the instrument 
/// HIFN  (reading of the Standard Event Register and checking it for error). All 
/// HIFN  driver functions are using state checking. Switch this function to ON 
/// HIFN  when debug your application. For better bus throughput and instrument 
/// HIFN  performance switch it OFF. As default is state checking switched to ON!
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control selects ON or OFF instrument state checking.
ViStatus _VI_FUNC rsavpva_setCheckStatus(
	ViSession	instrSession,
	ViBoolean	state
)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rsavpva_SetAttributeViBoolean (instrSession, "",
                                                RS_ATTR_QUERY_INSTRUMENT_STATUS,
												state
                                                ), 2, "State");
Error:
    Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function switches the display on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvtx_init or 
/// HIPAR instrSession/rsvtx_InitWithOptions function.
/// HIPAR displayUpdate/Selects the local oscillator mode.
ViStatus _VI_FUNC rsavpva_ConfigureDisplayUpdate (ViSession instrSession,
                                                  ViBoolean displayUpdate)
{
    ViStatus	error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, VI_NULL));

	viCheckParm(rsavpva_SetAttributeViBoolean (instrSession, "",
        RSAVPVA_ATTR_DISPLAY_UPDATE, displayUpdate), 2, "Display Update");
	
Error:
    Rs_UnlockSession(instrSession, VI_NULL);    
	return error;
}


/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsavpva_GetError (ViSession instrSession, 
                                     ViStatus *errorCode, 
                                     ViInt32 bufferSize,
                                     ViChar description[])  
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == VI_NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }

    else
        {
            if (sessionProperties == NULL)
            {
                (void) Rs_UnlockSession (instrSession, &locked);
                return 1024;
            }
            else
            {
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
                checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
            }
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpva_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rsavpva_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        if (description == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
ViStatus _VI_FUNC rsavpva_ClearError (ViSession instrSession)                                                        
{                                                                                                           
    return Rs_ClearErrorInfo (instrSession);                                                                             
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsavpva_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr (Rs_WriteInstrData (instrSession, writeBuffer));
    
    checkErr (rsavpva_CheckStatus (instrSession));
 
Error:   
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsavpva_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsavpva_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rsavpva_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsavpva_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rsavpva_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsavpva_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rsavpva_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsavpva_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==VI_NULL)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rsavpva_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
ViStatus _VI_FUNC rsavpva_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rsavpva_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/// HIFN This function performs all of the drivers clean-up operations
/// HIFN except for closing the RS session.  This function is called by
/// HIFN rsavpva_close or by an RS class driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpva_init or 
/// HIPAR instrSession/rsavpva_InitWithOptions function.
ViStatus _VI_FUNC rsavpva_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViSession   io      = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, (ViInt32) sizeof (io), &io));

Error:

    (void) Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, VI_NULL);
    
    if(io != 0)
        {
        ViSession   rmSession = VI_NULL;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession));
        
        (void) viClose (io);
        (void) viClose (rmSession);
        }
    
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
